#ifndef THUMBNAILMANAGER_H
#define THUMBNAILMANAGER_H

#include <QObject>
#include <KWindowSystem>
#include <QPixmap>
#include <QMap>
#include <QMutex>
#include <QSettings>

class ThumbnailManager : public QObject
{
    Q_OBJECT
private:
    explicit ThumbnailManager(QObject *parent = nullptr);

public:
    static ThumbnailManager * getIns();

    QPixmap getThumbnail(WId id);

    QPixmap getWallPaper();
signals:
    void windowAdded(WId id);
    void windowRemoved(WId id);
    void windowChanged(WId window, NET::Properties prop, NET::Properties2 prop2);
public slots:
    void onKwinWindowAdd(WId id);
    void onKwinWindowRemove(WId id);
    void onKwinWindowChanged(WId id, NET::Properties p1, NET::Properties2 p2);

private slots:
    void onBackgroundChanged();

private:
    void removeThumbnail(WId id);
    void updateThumbnail(WId id);
    QString getBackgroundSettingPath();

private:
    QMutex m_thumbnailsLock;
    QMap<WId, QPixmap> m_thumbnails;

    QString m_wallPaperPath;
    QPixmap m_wallPaper;
};
#endif // THUMBNAILMANAGER_H
