#include "newstartwidget.h"
#include <QPainter>
#include <QFile>
#include <QGraphicsDropShadowEffect>
#include <QDebug>
#include <QBitmap>
#include "desktopitem.h"
namespace NewStart {

#define TEXT_HEIGHT 30
#define SPACE_BETWEEN 10
#define PEN_FRAME 2
#define MAX_LENGTH 400
#define MAX_WIDTH 600
#define ICON_LENGTH 80
#define ICON_WIDTH 100
#define DESKTOPSIZE QSize(180,130)


Window * selectwindow = nullptr;
Desktop* selectdesktop = nullptr;

QPixmap QPixmapToRound(const QPixmap &src, int radius)
{
    if(src.isNull())
    {
        return QPixmap();
    }

    QSize size(src.width(), src.height());
    QBitmap mask(size);
    QPainter painter(&mask);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    painter.fillRect(0,0,size.width(),size.height(),Qt::white);
    painter.setBrush(QColor(0,0,0));
    painter.drawRoundedRect(0,0,size.width(),size.height(),radius,radius);

    QPixmap image = src.scaled(size,Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation);
    image.setMask(mask);
    return image;
}


PluButton::PluButton(QWidget * parent)
    : QWidget(parent)
{
    title = tr("New Desktop");
    setFixedSize(DESKTOPSIZE);
    connect(this,SIGNAL(PluClick()),&desktopmanager::instance(),SLOT(adddesktop()));
}

void PluButton::paintEvent(QPaintEvent *event)
{

    QPainter painter(this);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);//消锯齿
    painter.beginNativePainting();
    QRect textRc = QRect(5,0,150,20);
    QColor textcolor = QColor("#ffffff");
    if(hover || click){

    } else {
        textcolor.setAlphaF(0.5);
    }
    QFont f;
    f.setPixelSize(14);
    painter.setFont(f);
    painter.setPen(textcolor);
    painter.setBrush(Qt::NoBrush);
    QTextOption option(Qt::AlignLeft);
    option.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
    painter.drawText(textRc, title,option);

    QColor lineColor;
    if (click) {
        lineColor= QColor("#2c9dfc");
        painter.setPen(QPen(lineColor,2));
    } else if (hover) {
        lineColor= QColor("#ffffff");
        lineColor.setAlphaF(0.8);
        painter.setPen(QPen(lineColor,2));
    } else {
        lineColor= QColor("#ffffff");
        lineColor.setAlphaF(0.3);
        painter.setPen(QPen(lineColor,1));
    }
    QRect rect = QRect(0,30,180,100);
    painter.setBrush(Qt::NoBrush);
    painter.drawRoundedRect(rect,6,6);

    painter.setPen(QPen(Qt::white,2));
    painter.setBrush(Qt::NoBrush);
    painter.drawLine(75,80,105,80);
    painter.drawLine(90,65,90,95);

}

CloseButton::CloseButton(QWidget * parent,WId wid,bool desktop)
    : QWidget(parent)
{
    m_wid = wid;
    m_desktop = desktop;
    setFixedSize(12,12);
    //setVisible(true);
}

void CloseButton::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);//消锯齿
    painter.beginNativePainting();

    QColor color = QColor("#ffffff");
    if(pressed){
        color.setAlphaF(0.4);
    } else if (hover) {
        color.setAlphaF(1.0);
    } else {
        color.setAlphaF(0.6);
    }

    painter.setPen(color);
    painter.setBrush(Qt::NoBrush);
    painter.drawLine(0,0,12,12);
    painter.drawLine(12,0,0,12);
}


Desktop::Desktop(bool isdesktop, int id ,QWidget *parent) :
    QWidget(parent),
    m_desktop(id)
{
    m_selected = false;
    m_titleVisible = true;


    closebutton = new CloseButton(this,id,isdesktop);
    connect(this,SIGNAL(mouseenter()),closebutton,SLOT(mouseenter()));
    connect(this,SIGNAL(mouseleave()),closebutton,SLOT(mouseleave()));
    connect(closebutton,SIGNAL(CloseClick()),this,SLOT(removedesktop()));

    if (NewStart::KWinOpt::instance().getdesktopnum() ==1){
        closebutton->setVisible(false);
    }

    setAcceptDrops(true);
}

Desktop::~Desktop()
{
    if (this == selectdesktop) {
        selectdesktop = nullptr;
    }
}

void Desktop::setPixmap(const QPixmap pixmap)
{
    m_pixmap = pixmap;
    setFixedSize(DESKTOPSIZE);
    repaint();
}

void Desktop::setselected()
{
    if(selectdesktop) {
        if (this == selectdesktop) {
            return;
        }
        selectdesktop->selectChanged();
    }
    selectdesktop = this;
    m_selected = true;
    NewStart::desktopmanager::instance().setSelectedDesktop(m_desktop);
    emit desktopChanged(m_desktop);
    repaint();
}
void Desktop::cancleseelect()
{
    m_selected = false;
    if (this == selectdesktop) {
        selectdesktop = nullptr;
    }
    repaint();
}

void Desktop::initwindowspic()
{
    QSize desktopsize(QApplication::desktop()->screenGeometry().width(), QApplication::desktop()->screenGeometry().height());
    auto windowlist = desktopmanager::instance().getdesktopbyid(m_desktop)->getwindowslist();
    for( auto it=windowlist.constBegin(); it!=windowlist.constEnd(); ++it)
    {
        QPixmap pic = QPixmap::fromImage(it.value()->desktopimg());
        if(pic.isNull()) continue;

        int width = pic.width() * ICON_WIDTH / desktopsize.width() ;
        int height = width * pic.height() / pic.width();

        QSize tsize(width,height);
        m_piclist.insert(it.key(),pic.scaled(tsize,Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation));
    }
}



void Desktop::setText(const QString text)
{
    m_title = text;
}

void Desktop::backgrounddel(WId wid) {
    auto iter = m_piclist.find(wid);
    if(iter != m_piclist.end()){
        m_piclist.remove(wid);
    }
    repaint();
}
void Desktop::backgroundadd(WId wid)
{

    QSize desktopsize(QApplication::desktop()->screenGeometry().width(), QApplication::desktop()->screenGeometry().height());
    auto window = desktopmanager::instance().getdesktopbyid(m_desktop)->findwindowbywid(wid);

    QPixmap pic = QPixmap::fromImage(window->desktopimg());
    int width = pic.width() * ICON_WIDTH / desktopsize.width() ;
    int height = width * pic.height() / pic.width();
    QSize tsize(width,height);
    m_piclist.insert(wid,pic.scaled(tsize,Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation));
    repaint();
}

void Desktop::paintEvent(QPaintEvent *event)
{
    startx = 30;
    starty = 30;
    QPainter painter(this);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);//消锯齿
    painter.beginNativePainting();
    QRect rc = this->rect();

    //画图形
    if(!m_pixmap.isNull())
    {
        QRect pixmapRc = getPixmapRect();
        QBrush brush;
        brush.setTexture(m_pixmap);
        painter.setPen(Qt::NoPen);
        painter.setBrushOrigin(0,TEXT_HEIGHT);
        painter.setBrush(brush);
        painter.drawRoundedRect(pixmapRc,6,6);

        if (shadow && (!m_selected)){
            painter.setPen(Qt::NoPen);
            QColor color = QColor("#000000");
            color.setAlphaF(0.4);
            painter.setBrush(color);
            painter.drawRoundedRect(pixmapRc,6,6);
        }
        if(m_selected){
            painter.setPen(QPen(QColor("#2c9dfc"),PEN_FRAME));
            painter.setBrush(Qt::NoBrush);
            painter.drawRoundedRect(pixmapRc,6,6);
        }

        for( auto it=m_piclist.constBegin(); it!=m_piclist.constEnd(); ++it)
        {
            QRect pixmapRc = getpicRect();
            painter.setPen(Qt::NoPen);
            painter.setBrush(Qt::NoBrush);
            QPixmap tmppix = it.value();
            painter.drawPixmap(pixmapRc, tmppix);
        }

    }


    QRect textRc = getTextRect();
    QColor textcolor = QColor("#ffffff");
    QFont f;
    f.setPixelSize(14);
    if (shadow && (!m_selected)){
        textcolor.setAlphaF(0.5);
    }
    painter.setFont(f);
    painter.setPen(textcolor);
    painter.setBrush(Qt::NoBrush);
    QTextOption option(Qt::AlignLeft);
    option.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
    painter.drawText(textRc, m_title,option);

    closebutton->setGeometry(rc.width()-20,10,20,20);


    painter.endNativePainting();
}

void Desktop::mouseDoubleClickEvent(QMouseEvent *event)
{
    emit desktopselectd(m_desktop);
}

void Desktop::mousePressEvent(QMouseEvent *event)
{
    if(selectdesktop) {
        if (this == selectdesktop) {
            return;
        }
        selectdesktop->selectChanged();
    }
    selectdesktop = this;
    m_selected = true;
    NewStart::desktopmanager::instance().setSelectedDesktop(m_desktop);
    emit desktopChanged(m_desktop);
    if (KWinOpt::instance().getdesktopnum() !=1){
        emit mouseenter();
    }
    repaint();
    //emit desktopselectd(m_desktop);
}

void Desktop::mouseReleaseEvent(QMouseEvent *event)
{
}

void Desktop::dragEnterEvent(QDragEnterEvent  * e){
    //处理是否接受拖动事件。
    e->accept();  			//接受拖动进入事件
 }

 void Desktop::dropEvent(QDropEvent  * e){
    //处理拖动中的数据
    QString str = e->mimeData()->text();
    QStringList list = str.split(",");
    QString a = list[0]; //desktop
    QString b = list[1]; //window

    int tmpdesktop = a.toInt();
    WId temwindow = b.toUInt();

    if (tmpdesktop == m_desktop) {
        e->setDropAction(Qt::IgnoreAction);
        e->accept();
        return ;
    }
    e->setDropAction(Qt::MoveAction);
    e->accept();
    desktopmanager::instance().movewindowfromto(temwindow,tmpdesktop,m_desktop);
 }

void Desktop::enterEvent(QEvent *)                      //进入QWidget瞬间事件
{
    shadow = false;
    //emit mouseenter();
    repaint();
}
void Desktop::leaveEvent(QEvent *)                      //离开QWidget瞬间事件
{
    shadow = true;
    //emit mouseleave();
    repaint();
}

QRect Desktop::getPixmapRect()
{
    QRect rc = this->rect();
    QRect imageRc = rc.adjusted(0,TEXT_HEIGHT,0,0);
    return imageRc;
}

QRect Desktop::getpicRect()
{
    QRect rc = this->rect();
    QRect imageRc(startx,starty,ICON_WIDTH,ICON_LENGTH);
    startx = startx + 20;
    if (startx > 70) { startx = 30; }
    starty = starty + 20;
    if (starty > 70) { startx = 30; }
    return imageRc;
}

QRect Desktop::getTextRect()
{
    QRect rc = this->rect();
    QRect textrc(4,2,rc.width()-10,TEXT_HEIGHT);
    return textrc;
}

Window::Window(WId wid,int d_id,QWidget *parent) :
    QWidget(parent),
    m_wid(wid),
    desktopid(d_id),
    m_select(false)
{
    closebutton = new NewStart::CloseButton(this,wid,false);
    closebutton->setVisible(false);
    setAcceptDrops(true);
    connect(this,SIGNAL(mouseenter()),closebutton,SLOT(mouseenter()));
    connect(this,SIGNAL(mouseleave()),closebutton,SLOT(mouseleave()));
    connect(closebutton,SIGNAL(CloseClick()),this,SLOT(removewindow()));

    QGraphicsDropShadowEffect * shadoweffect = new QGraphicsDropShadowEffect(this);
    shadoweffect->setOffset(0,0);
    QColor scolor = QColor("#000000");
    scolor.setAlphaF(0.3);
    shadoweffect->setColor(scolor);
    shadoweffect->setBlurRadius(3);
    setGraphicsEffect(shadoweffect);
}

Window::~Window()
{
    if (this == selectwindow) {
        selectwindow = nullptr;
    }
}

void Window::mousePressEvent(QMouseEvent *event)
{
    pos=event->pos();
    m_select = true;
    repaint();
}

void Window::mouseMoveEvent(QMouseEvent *e)
{
    //若拖动的距离大于5个像素则启用拖放
    if((e->pos()-pos).x()>=5||(e->pos()-pos).y()>=5) {
        setVisible(false);
        //在该事件中启动拖放
        QDrag *dg=new QDrag(this);
        QPixmap m_pixmap = QPixmap(m_map);
        dg->setPixmap(m_pixmap.scaled(100,100,Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation));
        dg->setHotSpot(QPoint(50,50));

        QMimeData *md=new QMimeData;
        QString msg = QString::number(desktopid) + "," + QString::number(m_wid);
        md->setText(msg);    			//拖放时传递的数据
        dg->setMimeData(md);   			//设置拖动的数据
        Qt::DropAction action = dg->exec(Qt::MoveAction,Qt::MoveAction);

        if (action != Qt::MoveAction) {
            setVisible(true);
            repaint();
        }
    }
}

void Window::selectWindow()
{
    if(selectwindow) {
        if (this == selectwindow) {
            return;
        }
        selectwindow->selectChanged();
    }
    selectwindow = this;
    m_select = true;

    repaint();
}

void Window::enterEvent(QEvent *)                      //进入QWidget瞬间事件
{
    emit mouseenter();
    selectWindow();
}

void Window::leaveEvent(QEvent *)                      //离开QWidget瞬间事件
{
    emit mouseleave();
    repaint();
}
void Window::mouseReleaseEvent(QMouseEvent *e)
{

    if((e->pos()-pos).x()>=5||(e->pos()-pos).y()>=5){

    } else {
        emit activewindow(m_wid);
    }
    update();
}

void Window::setpixmap(QPixmap pic,WId wid,QString str)
{
    m_wid = wid;
    m_title = str;

    //根据宽和高的比率大小来判断以宽（或高）等比例缩放图片到允许的最大尺寸
    float ratioPic = pic.width()*1.0 / pic.height();
    //图片最大高度以MAX_LENGTH-TEXT_HEIGHT计算
    float ratioBase = MAX_WIDTH *1.0 / (MAX_LENGTH-TEXT_HEIGHT) ;
    int fixW = 0;
    int fixH = 0;
    //若图片的比率大，则判断pic的宽度与最大宽度
    if(ratioPic > ratioBase)
    {
        //图片宽度大于最大宽度，计算缩放后的尺寸
        if(MAX_WIDTH < pic.width())
        {
            fixW = MAX_WIDTH;
            fixH = MAX_WIDTH / ratioPic;
        }
        //小于最大宽度，不需要缩放
        else {
            fixW = pic.width();
            fixH = pic.height();
        }
    } else {//若图片的比率小，则判断pic的高度与最大高度
        //图片高度大于最大高度，计算缩放后的尺寸
        if(MAX_LENGTH < pic.height())
        {
            fixW = (MAX_LENGTH-TEXT_HEIGHT) * ratioPic;
            fixH = (MAX_LENGTH-TEXT_HEIGHT) ;
        }
        //小于最大高度，不需要缩放
        else {
            fixW = pic.width();
            fixH = pic.height();
        }
    }
    //图片缩放后的尺寸
    QSize tsize(fixW,fixH);
    m_map = pic.scaled(tsize,Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation);

    //window的尺寸，需要加文字标题的高度
    m_size = tsize + QSize(0,TEXT_HEIGHT);
    setFixedSize(m_size);
}

void Window::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    painter.beginNativePainting();
    QRect rc = this->rect();

    QRect shadowrc = rc.adjusted(0,TEXT_HEIGHT,0,0);
    QRect imageRc = QRect(2,TEXT_HEIGHT+2,rc.width()-4,rc.height()-TEXT_HEIGHT-4);
    QBrush brush;
    brush.setTexture(m_map);
    painter.setPen(Qt::NoPen);
    painter.setBrushOrigin(0,TEXT_HEIGHT);
    painter.setBrush(brush);
    painter.drawRoundedRect(imageRc,6,6);

    if(m_select){
        painter.setPen(QPen(QColor("#2c9dfc"),3));
        painter.setBrush(Qt::NoBrush);
        painter.drawRoundedRect(imageRc,6,6);

        QRect textRc(4,2,rc.width()/2,TEXT_HEIGHT);
        painter.setPen(Qt::white);
        painter.setBrush(Qt::NoBrush);
        QTextOption option(Qt::AlignLeft);
        option.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
        QString str = m_title.left(40);

        painter.drawText(textRc, str,option);
    }

    closebutton->setGeometry(rc.width()-20,10,20,20);
}


}
