#ifndef IMAGEITEMWIDGET_H
#define IMAGEITEMWIDGET_H

#include <QWidget>
#include <QPainter>
#include <QLabel>
#include <QDebug>
#include "desktopitem.h"
namespace NewStart {

class PluButton : public QWidget
{
    Q_OBJECT
public:
    PluButton(QWidget * parent) ;
    void setSelect(){
        hover = true;
        repaint();
    }
    void cancleSelect(){
        hover = false;
        repaint();
    }
    void Select(){
        emit PluClick();
    }
signals:
    void PluClick();
protected:
    virtual void paintEvent(QPaintEvent *event);
    virtual void mousePressEvent(QMouseEvent *event){
        click = true;

        repaint();
    }
    virtual void mouseReleaseEvent(QMouseEvent *event){
        click = false;
        emit PluClick();
        repaint();
    }
    virtual void enterEvent(QEvent *){
        hover = true;
        repaint();
    }
    virtual void leaveEvent(QEvent *){
        hover = false;
        repaint();
    }

private:
    QSize size;
    bool hover {false};
    bool click {false};
    QString title;

};


class CloseButton : public QWidget
{
    Q_OBJECT
public:
    CloseButton(QWidget * parent,WId wid,bool desktop) ;
signals:
    void CloseClick();
protected:
    virtual void paintEvent(QPaintEvent *event);
    virtual void mousePressEvent(QMouseEvent *event){
        pressed = true;
        repaint();

    }
    virtual void mouseReleaseEvent(QMouseEvent *event){
        pressed = false;
        repaint();
        emit CloseClick();
    }
    virtual void enterEvent(QEvent *){
        hover = true;
        repaint();
    }
    virtual void leaveEvent(QEvent *){
        hover = false;
        repaint();
    }
private slots:
    void onclose() {emit CloseClick();}
    void mouseenter() { setVisible(true); }
    void mouseleave() { setVisible(false); }
private:
    bool m_desktop;
    WId m_wid;
    bool pressed {false};
    bool hover {false};
};

class Desktop : public QWidget
{
    Q_OBJECT
public:
    explicit Desktop(bool isdesktop, int id ,QWidget *parent = 0);
    ~Desktop();

    void setPixmap(const QPixmap pixmap);
    void setText(const QString text);
    void initwindowspic();
    void setselected();
    void cancleseelect();
    bool isSelected(){
        return m_selected;
    }
    int getDesktopid(){
        return m_desktop;
    }

signals:
    void desktopChanged(int id);
    void desktopselectd(int id);
    void desktopdelete(int id);
    void mouseenter();
    void mouseleave();

private slots:
    void backgroundadd(WId);
    void backgrounddel(WId);
    void removedesktop() { emit desktopdelete(m_desktop);}

protected:
    virtual void paintEvent(QPaintEvent *event);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);
    virtual void dragEnterEvent(QDragEnterEvent  * e);
    virtual void dropEvent(QDropEvent  * e);
    virtual void enterEvent(QEvent *);
    virtual void leaveEvent(QEvent *);
    virtual void mouseDoubleClickEvent(QMouseEvent *event);
private:
    QRect getPixmapRect();
    QRect getTextRect();
    QRect getpicRect();
    void selectChanged(){
        m_selected = false;
        repaint();
    }
private:
    bool m_selected;        //是否选中
    QString m_title;         //图片名称
    bool m_titleVisible;     //名称可见
    QPixmap m_pixmap;       //图片
    QSize desktopsize;
    int m_desktop;

    QHash<WId,QPixmap> m_piclist;
    CloseButton * closebutton;
    bool shadow {true};
    int startx ,starty;
};

class Window : public QWidget
{
    Q_OBJECT
public:
    Window(WId wid,int d_id,QWidget *parent = nullptr) ;
    ~Window();

    void setpixmap(QPixmap pic,WId Wid,QString str);
    void selectChanged(){
        m_select = false;
        repaint();
    }

    void selectWindow();
    bool isselect(){
        return m_select;
    }
    WId getWindwonid(){
        return m_wid;
    }

signals:
    void mouseenter();
    void mouseleave();
    void activewindow(WId id);
    void deletewindow(WId id);
    void Refresh(int id);
private slots:
    void removewindow() { emit deletewindow(m_wid);}
protected:
    virtual void paintEvent(QPaintEvent *event);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseMoveEvent(QMouseEvent *e);
    virtual void mouseReleaseEvent(QMouseEvent *event);
    virtual void enterEvent(QEvent *);
    virtual void leaveEvent(QEvent *);

private:
    QPoint pos;
    QPixmap m_map;
    QSize m_size;
    WId m_wid;
    int desktopid;
    QString m_title;
    bool m_select;

    NewStart::CloseButton * closebutton;
};




}
#endif // IMAGEITEMWIDGET_H
