#include "displaymainmenu.h"
#include "ui_displaymainmenu.h"
#include <XdgIcon>
#include <QProcess>
#include <QDir>
#include <QBitmap>
#include <QTimer>
#include <QKeyEvent>
#include "pinyinhandle.h"
#include <appmanager.h>
#include <QPixmap>
#include <QDebug>

//RGBA 转为RGB
//QColor rgbaToRgb(QColor color)
//{
//    qreal bgColor = 0.1; //0:black 1:white

//    qreal r = bgColor * (1.0- color.alphaF()) + color.redF() * color.alphaF();
//    qreal g = bgColor * (1.0- color.alphaF()) + color.greenF() * color.alphaF();
//    qreal b = bgColor * (1.0- color.alphaF()) + color.blueF() * color.alphaF();

//    return QColor(r * 255, g * 255, b * 255);
//}

DisplayMainMenu::DisplayMainMenu(QWidget *parent):
    QWidget(parent),
    ui(new Ui::DisplayMainMenu),
    mShowPowerMenu(false),
    mCommonCategory(nullptr),
    mInputMethodActive(false)
{
    ui->setupUi(this);
    mMenuLayout = new QVBoxLayout();
    mMenuLayout->setMargin(0);
    mMenuLayout->setContentsMargins(0,0,0,0);
    mMenuLayout->setSpacing(2);
    mMenuLayout->addStretch();
    ui->scrollAreaWidgetContents->setFocusPolicy(Qt::NoFocus);
    ui->scrollAreaWidgetContents->setLayout(mMenuLayout);

//    mTopGradLabel = new GradientLabel(this);
//    mTopGradLabel->raise();
//    mBottomGradLabel = new GradientLabel(this);
//    mBottomGradLabel->raise();
//    ui->btn_extend->raise();
//    ui->btn_switch->raise();
//    ui->btn_power->raise();
//    ui->btn_settting->raise();
//    ui->lineEdit_search->raise();

    connect(ui->lineEdit_search, &QLineEdit::textChanged, this, &DisplayMainMenu::onSearchTextChanged);
    connect(ui->lineEdit_search, &QLineEdit::returnPressed, this, &DisplayMainMenu::onLineeditReturnPressed);
    connect(ui->btn_switch, &QToolButton::clicked, this, &DisplayMainMenu::onSwitchClicked);
    connect(ui->btn_switch_common, &QToolButton::clicked, this, &DisplayMainMenu::onSwitchCommonClicked);
    connect(ui->btn_user, &QToolButton::clicked, this, &DisplayMainMenu::onUserClicked);
    connect(ui->btn_computer, &QToolButton::clicked, this, &DisplayMainMenu::onComputerClicked);
    connect(ui->btn_softwarecenter, &QToolButton::clicked, this, &DisplayMainMenu::onSoftwareCenterClicked);
    connect(ui->btn_settting, &QToolButton::clicked, this, &DisplayMainMenu::onSettingClicked);
    connect(ui->btn_power, &QToolButton::clicked, this, &DisplayMainMenu::onPowerClicked);
    connect(ui->btn_extend, &QToolButton::clicked, this, [](){
        AppManager::getIns()->setMenuDisplayMode(DisplayFull);
    });

    ui->lineEdit_search->installEventFilter(this);

    mPowerMenu = new PowerMenu(this);
    connect(mPowerMenu, &PowerMenu::signalAnimFinished, this, &DisplayMainMenu::onPowerMenuAnimFinished);
    mPowerMenu->hide();

    connect(AppManager::getIns(), &AppManager::menuChanged, this, &DisplayMainMenu::onMenuChanged);
    connect(AppManager::getIns(), &AppManager::commonAppDataChanged, this, &DisplayMainMenu::onCommonAppChanged);
    connect(AppManager::getIns(), &AppManager::menuCategoryChanged, this, &DisplayMainMenu::onMenuCategoryChanged);
}

DisplayMainMenu::~DisplayMainMenu()
{
    delete ui;
}

void DisplayMainMenu::restoreUI()
{
    ui->lineEdit_search->setText("");
    ui->lineEdit_search->setFocus();
    ui->btn_user->show();
    ui->btn_computer->show();
    ui->btn_softwarecenter->show();
    ui->btn_settting->show();

    ui->btn_switch_common->setChecked(false);

    mShowPowerMenu = false;
    mPowerMenu->hide();

    switchToMenuPage();
}

bool DisplayMainMenu::eventFilter(QObject *watched, QEvent *event)
{
    if(watched == ui->lineEdit_search)
    {
        if(event->type() == QEvent::KeyPress && !mInputMethodActive)
        {
            QKeyEvent * keyEvent = static_cast<QKeyEvent*>(event);
            if(keyEvent && keyEvent->key() == Qt::Key_Up)
            {
                event->accept();
                focusLastItem();
                return true;
            }
            if(keyEvent && keyEvent->key() == Qt::Key_Down)
            {
                event->accept();
                focusFirstItem();
                return true;
            }
        }
        else if(event->type() == QEvent::InputMethod)
        {
            QInputMethodEvent * keyEvent = static_cast<QInputMethodEvent*>(event);

            //keyEvent->preeditString() 若该字符串不为空说明输入法被激活，否则没有激活输入法
            mInputMethodActive = !keyEvent->preeditString().isEmpty();
        }
    }
    return QWidget::eventFilter(watched, event);
}

void DisplayMainMenu::onMenuChanged()
{
    if(!ui->btn_switch_common->isChecked())
    {
        switchToMenuPage();
    }
}

void DisplayMainMenu::onCommonAppChanged()
{
    if(ui->btn_switch_common->isChecked())
    {
        switchToCommonPage();
    }
}

//void DisplayMainMenu::setGradientColor(QColor color)
//{
//    if(mTopGradLabel) {
//        int height = ui->verticalLayout->contentsMargins().top()+12;
//        QRect rect = QRect(0,0, this->width(), height);
//        mTopGradLabel->setDirection(GradientLabel::TopToBottom);
//        mTopGradLabel->setGeometry(rect);
//        QPixmap pixmap(rect.size());
//        pixmap.fill(color);
//        mTopGradLabel->setPixmap(pixmap);
//    }
//    if(mBottomGradLabel) {
////        color.setAlpha(255);
//        int height = ui->verticalLayout->contentsMargins().bottom() +
//                ui->verticalLayout->spacing() + ui->lineEdit_search->height()+20;
//        mBottomGradLabel->setDirection(GradientLabel::BottomToTop);
//        mBottomGradLabel->setGeometry(0,this->height() - height, this->width(), height);

//        QPixmap pixmap(this->width(),height);
//        pixmap.fill(color);
//        mBottomGradLabel->setPixmap(pixmap);
//    }
//}

void DisplayMainMenu::onPowerMenuAnimFinished()
{
    if(!mShowPowerMenu)
    {
        ui->btn_user->setVisible(!mShowPowerMenu);
        ui->btn_computer->setVisible(!mShowPowerMenu);
        ui->btn_softwarecenter->setVisible(!mShowPowerMenu);
        ui->btn_settting->setVisible(!mShowPowerMenu);
    }
}

void DisplayMainMenu::onSearchTextChanged(const QString &filter)
{
    if(mCommonCategory)
    {
        mCommonCategory->setFilter(filter);
    }

    QStringList categorys = mAllCategories.keys();
    categorys.sort(Qt::CaseInsensitive);
    while(categorys.count())
    {
        QString category = categorys.takeFirst();

        MainMenuCategoryItem * categoryItem = mAllCategories.value(category);
        categoryItem->setFilter(filter);
    }
}

void DisplayMainMenu::onLineeditReturnPressed()
{
    focusFirstItem();
}

void DisplayMainMenu::onMenuCategoryChanged()
{
    if(!ui->btn_switch_common->isChecked())
    {
        this->switchToMenuPage();
    }
}

void DisplayMainMenu::onSwitchClicked()
{
    if(ui->btn_switch_common->isChecked())
    {
        //若常用按下，隐藏常用分类界面，显示应用分类列表界面
        ui->btn_switch_common->setChecked(false);
        this->switchToMenuPage();
    }
    else
    {
        //若常用没有按下，切换应用显示分类
        AppManager::getIns()->switchMenuCategory();
    }
}

void DisplayMainMenu::onSwitchCommonClicked()
{
    if(ui->btn_switch_common->isChecked())
    {
        this->switchToCommonPage();
    }
    else
    {
        this->switchToMenuPage();
    }
}

void DisplayMainMenu::onSoftwareCenterClicked()
{
    QProcess::startDetached("/usr/sbin/start-softwarecenter");
    this->window()->hide();
}

void DisplayMainMenu::onUserClicked()
{
    QProcess::startDetached("nde-admin-user");
    this->window()->hide();
}

void DisplayMainMenu::onComputerClicked()
{
    QString command = QString::fromUtf8("nde-fileman %1").arg(QDir::homePath());
    QProcess::startDetached(command);
    this->window()->hide();
}

void DisplayMainMenu::onSettingClicked()
{
    QProcess::startDetached("nde-systemsettings");
    this->window()->hide();
}

void DisplayMainMenu::onPowerClicked()
{
    QPoint p(ui->btn_power->pos());
    p.ry() = p.y() - mPowerMenu->height();
    p.rx() = p.x() - (mPowerMenu->width() - ui->btn_power->width())/2;
    mPowerMenu->move(p);
    mPowerMenu->raise();

    mShowPowerMenu =!mShowPowerMenu;
    switchPowerMenu(mShowPowerMenu);
}

void DisplayMainMenu::switchToMenuPage()
{
    clearLayout();

    QHash<QString, QList<XdgDesktopFile> > mapApps = AppManager::getIns()->getApps();
    for(QString category : mapApps.keys())
    {
        QList<XdgDesktopFile> list = mapApps.value(category);
        for(int i = 0; i < list.count(); i++)
        {
            XdgDesktopFile desktopFile = list.at(i);
            MainMenuApplicationItem* item = new MainMenuApplicationItem(
                        desktopFile,ui->scrollAreaWidgetContents);
            connect(item, &MainMenuApplicationItem::signalClicked, [this](){
                this->topLevelWidget()->hide();});
            connect(item, &MainMenuApplicationItem::signalStartDrag, [this](){
                this->topLevelWidget()->hide();});
            connect(item, &MainMenuApplicationItem::signalFocusNext, [this](){this->focusNextItem();});
            connect(item, &MainMenuApplicationItem::signalFocusPrvi, [this](){this->focusPrivItem();});
            connect(item, &MainMenuApplicationItem::signalFocusSearch, [this](){ui->lineEdit_search->setFocus();});
            connect(item, &MainMenuApplicationItem::signalAddToDesktop,
                    this, &DisplayMainMenu::signalAddToDesktop);
            connect(item, &MainMenuApplicationItem::signalAddToQuickLaunch,
                    this, &DisplayMainMenu::signalAddToQuickLaunch);

            QHash<QString, MainMenuCategoryItem*>::iterator it = mAllCategories.find(category);
            if(it != mAllCategories.end())
            {
                it.value()->addItem(item);
            }
            else
            {
                MainMenuCategoryItem * categoryItem = new MainMenuCategoryItem(category, ui->scrollAreaWidgetContents);
                categoryItem->addItem(item);
                mAllCategories.insert(category, categoryItem);
            }
        }
    }

    addToLayout();
}

void DisplayMainMenu::switchToCommonPage()
{
    clearLayout();

    QList<XdgDesktopFile> commonApp =  AppManager::getIns()->getCommonApps(DisplayNormal);
    for(XdgDesktopFile common : commonApp)
    {
        MainMenuApplicationItem* item = new MainMenuApplicationItem(
                    common,ui->scrollAreaWidgetContents);
        connect(item, &MainMenuApplicationItem::signalClicked, [this](){
            this->topLevelWidget()->hide();});
        connect(item, &MainMenuApplicationItem::signalStartDrag, [this](){
            this->topLevelWidget()->hide();});
        connect(item, &MainMenuApplicationItem::signalFocusNext, [this](){this->focusNextItem();});
        connect(item, &MainMenuApplicationItem::signalFocusPrvi, [this](){this->focusPrivItem();});
        connect(item, &MainMenuApplicationItem::signalFocusSearch, [this](){ui->lineEdit_search->setFocus();});
        connect(item, &MainMenuApplicationItem::signalAddToDesktop,
                this, &DisplayMainMenu::signalAddToDesktop);
        connect(item, &MainMenuApplicationItem::signalAddToQuickLaunch,
                this, &DisplayMainMenu::signalAddToQuickLaunch);

        if(!mCommonCategory)
        {
            mCommonCategory = new MainMenuCategoryItem(tr("Common"), ui->scrollAreaWidgetContents);
        }
        mCommonCategory->addItem(item);
    }

    addToLayout();
}

void DisplayMainMenu::switchPowerMenu(bool showPowerMenu)
{
    if(showPowerMenu)
    {
        ui->btn_user->setVisible(!showPowerMenu);
        ui->btn_computer->setVisible(!showPowerMenu);
        ui->btn_softwarecenter->setVisible(!showPowerMenu);
        ui->btn_settting->setVisible(!showPowerMenu);
        mPowerMenu->setVisible(showPowerMenu);
    }

    mPowerMenu->expend(showPowerMenu);
}

void DisplayMainMenu::addToLayout()
{
    //
    this->setTabOrder(ui->lineEdit_search, ui->btn_switch);
    this->setTabOrder(ui->btn_switch, ui->btn_switch_common);

    QWidget * priv = ui->btn_switch_common;
    QList<ToolButton*> btns = mPowerMenu->getBtns();
    for(ToolButton* btn : btns)
    {
        this->setTabOrder(priv, btn);
        priv = btn;
    }
    this->setTabOrder(priv, ui->btn_user);

    this->setTabOrder(ui->btn_user, ui->btn_computer);
    this->setTabOrder(ui->btn_computer, ui->btn_softwarecenter);
    this->setTabOrder(ui->btn_softwarecenter, ui->btn_settting);
    this->setTabOrder(ui->btn_settting, ui->btn_power);

    priv = ui->btn_power;
    //
    if(mCommonCategory)
    {
        mMenuLayout->addWidget(mCommonCategory);

        QList<MainMenuApplicationItem*> items = mCommonCategory->getItems();
        QList<MainMenuApplicationItem*>::iterator it = items.begin();
        while(it != items.end())
        {
            this->setTabOrder(priv, *it);
            mMenuLayout->addWidget(*it);
            priv = *it;
            it++;
        }
    }

    QStringList categorys = mAllCategories.keys();
    categorys.sort(Qt::CaseInsensitive);
    while(categorys.count())
    {
        QString category = categorys.takeFirst();

        MainMenuCategoryItem * categoryItem = mAllCategories.value(category);
        mMenuLayout->addWidget(categoryItem);

        QList<MainMenuApplicationItem*> items = categoryItem->getItems();
        QList<MainMenuApplicationItem*>::iterator it = items.begin();
        while(it != items.end())
        {
            this->setTabOrder(priv, *it);
            mMenuLayout->addWidget(*it);
            priv = *it;
            it++;
        }
    }

    this->setTabOrder(priv, ui->btn_extend);
    this->setTabOrder(ui->btn_extend, ui->lineEdit_search);

    mMenuLayout->addStretch();
}

void DisplayMainMenu::clearLayout()
{
    if(mCommonCategory)
    {
        mMenuLayout->removeWidget(mCommonCategory);

        QList<MainMenuApplicationItem*> items = mCommonCategory->getItems();
        while(items.count())
        {
            MainMenuApplicationItem* item = items.takeFirst();
            mCommonCategory->removeItem(item);
            delete item;
        }
        delete mCommonCategory;
        mCommonCategory = nullptr;
    }

    QStringList categorys = mAllCategories.keys();
    categorys.sort(Qt::CaseInsensitive);
    while(categorys.count())
    {
        QString category = categorys.takeFirst();

        MainMenuCategoryItem * categoryItem = mAllCategories.take(category);
        mMenuLayout->removeWidget(categoryItem);

        QList<MainMenuApplicationItem*> items = categoryItem->getItems();
        while(items.count())
        {
            MainMenuApplicationItem* item = items.takeFirst();
            categoryItem->removeItem(item);
            delete item;
        }
        delete categoryItem;
    }

    while(mMenuLayout->count())
    {
        delete mMenuLayout->takeAt(0);
    }

    ui->lineEdit_search->setText("");
}

QList<MainMenuApplicationItem *> DisplayMainMenu::getCurSortItems()
{
    QList<MainMenuApplicationItem *> list;
    if(mCommonCategory)
    {
        QList<MainMenuApplicationItem*> items = mCommonCategory->getItems();
        QList<MainMenuApplicationItem*>::iterator it = items.begin();
        while(it != items.end())
        {
            if((*it)->isVisible()) {
                list.append(*it);
            }
            it++;
        }
    }

    QStringList categorys = mAllCategories.keys();
    categorys.sort(Qt::CaseInsensitive);
    while(categorys.count())
    {
        QString category = categorys.takeFirst();

        MainMenuCategoryItem * categoryItem = mAllCategories.value(category);

        QList<MainMenuApplicationItem*> items = categoryItem->getItems();
        QList<MainMenuApplicationItem*>::iterator it = items.begin();
        while(it != items.end())
        {
            if((*it)->isVisible()){
                list.append(*it);
            }
            it++;
        }
    }
    return list;
}

void DisplayMainMenu::focusFirstItem()
{
    QList<MainMenuApplicationItem*>  list = getCurSortItems();
    if(list.count() > 0)
    {
        MainMenuApplicationItem *item = list.at(0);
        item->setFocus();
        ui->scrollArea->ensureWidgetVisible(item);
    }
}

void DisplayMainMenu::focusLastItem()
{
    QList<MainMenuApplicationItem*>  list = getCurSortItems();
    if(list.count() > 0)
    {
        MainMenuApplicationItem *item = list.at(list.count()-1);
        item->setFocus();
        ui->scrollArea->ensureWidgetVisible(item);
    }
}

void DisplayMainMenu::focusNextItem()
{
    QList<MainMenuApplicationItem*>  list = getCurSortItems();
    MainMenuApplicationItem * next = 0;
    MainMenuApplicationItem * cur = 0;
    QList<MainMenuApplicationItem*>::iterator it = list.begin();
    while (it != list.end()) {
        if(cur){
            next = *it;
            break;
        }
        if(this->focusWidget() == *it)
        {
            cur = *it;
        }
        it++;
    }

    if(next)
    {
        next->setFocus();
        ui->scrollArea->ensureWidgetVisible(next);
    }
    else
    {
        ui->lineEdit_search->setFocus();
    }
}

void DisplayMainMenu::focusPrivItem()
{
    QList<MainMenuApplicationItem*>  list = getCurSortItems();
    MainMenuApplicationItem * prvi = 0;
    MainMenuApplicationItem * cur = 0;
    QList<MainMenuApplicationItem*>::iterator it = list.end();
    while (it != list.begin()) {
        it--;
        if(cur){
            prvi = *it;
            break;
        }
        if(this->focusWidget() == *it)
        {
            cur = *it;
        }
    }

    if(prvi)
    {
        prvi->setFocus();
        ui->scrollArea->ensureWidgetVisible(prvi);
    }
    else
    {
        ui->lineEdit_search->setFocus();
    }
}
