#include "powermenu.h"
#include <QCoreApplication>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QSettings>
#include <QDebug>


#define POWERBTNSIZE QSize(55,32)

PowerMenu::PowerMenu(QWidget *parent) :
    QWidget(parent),
    mExtend(false)
{
    this->setObjectName("PowerMenu");

    mLayout = new QVBoxLayout;
    mLayout->setMargin(0);
    mLayout->setContentsMargins(0,0,0,0);
    mLayout->setSpacing(0);

//    this->addAction(tr("睡眠"),this, &PowerMenu::onComputerSleep);
//    this->addAction(tr("切换"),this, &PowerMenu::onComputerSwitch);
//    this->addAction(tr("注销"),this, &PowerMenu::onComputerLogout);
//    this->addAction(tr("重启"),this, &PowerMenu::onComputerReboot);
//    this->addAction(tr("关机"),this, &PowerMenu::onComputerPowerOff);

    this->setFixedSize(POWERBTNSIZE.width(), POWERBTNSIZE.height()*5);

    ToolButton* btnSleep = new ToolButton(this);
    btnSleep->setToolButtonStyle(Qt::ToolButtonTextOnly);
    btnSleep->setText(tr("Sleep"));//睡眠
    btnSleep->setFixedSize(POWERBTNSIZE);
    btnSleep->setGeometry(QRect(QPoint(0,0), POWERBTNSIZE));
    connect(btnSleep, &ToolButton::clicked, this, &PowerMenu::onComputerSleep);
    mLayout->addWidget(btnSleep);
    QPropertyAnimation * btnSleepAnim = new QPropertyAnimation(btnSleep, "pos", btnSleep);
    connect(btnSleepAnim, &QPropertyAnimation::finished, this, &PowerMenu::onAnimFinished);
    btnSleepAnim->setDuration(200);
    btnSleepAnim->setStartValue(QPoint(0,POWERBTNSIZE.height()*5));
    btnSleepAnim->setEndValue(QPoint(0,0));
    mBtns.insert( btnSleep, btnSleepAnim);
    mBtnList.append(btnSleep);

    ToolButton* btnSwitch = new ToolButton(this);
    btnSwitch->setToolButtonStyle(Qt::ToolButtonTextOnly);
    btnSwitch->setText(tr("Switch"));//切换
    btnSwitch->setFixedSize(POWERBTNSIZE);
    btnSwitch->setGeometry(QRect(QPoint(0,0), POWERBTNSIZE));
    connect(btnSwitch, &ToolButton::clicked, this, &PowerMenu::onComputerSwitch);
    mLayout->addWidget(btnSwitch);
    QPropertyAnimation * btnSwitchAnim = new QPropertyAnimation(btnSwitch, "pos", btnSwitch);
    connect(btnSwitchAnim, &QPropertyAnimation::finished, this, &PowerMenu::onAnimFinished);
    btnSwitchAnim->setDuration(200);
    btnSwitchAnim->setStartValue(QPoint(0,POWERBTNSIZE.height()*5));
    btnSwitchAnim->setEndValue(QPoint(0,POWERBTNSIZE.height()*1));
    mBtns.insert( btnSwitch, btnSwitchAnim);
    mBtnList.append(btnSwitch);

    ToolButton* btnLogout = new ToolButton(this);
    btnLogout->setToolButtonStyle(Qt::ToolButtonTextOnly);
    btnLogout->setText(tr("Logout"));//注销
    btnLogout->setFixedSize(POWERBTNSIZE);
    btnLogout->setGeometry(QRect(QPoint(0,0), POWERBTNSIZE));
    connect(btnLogout, &ToolButton::clicked, this, &PowerMenu::onComputerLogout);
    mLayout->addWidget(btnLogout);
    QPropertyAnimation * btnLogoutAnim = new QPropertyAnimation(btnLogout, "pos", btnLogout);
    connect(btnLogoutAnim, &QPropertyAnimation::finished, this, &PowerMenu::onAnimFinished);
    btnLogoutAnim->setDuration(200);
    btnLogoutAnim->setStartValue(QPoint(0,POWERBTNSIZE.height()*5));
    btnLogoutAnim->setEndValue(QPoint(0,POWERBTNSIZE.height()*2));
    mBtns.insert( btnLogout, btnLogoutAnim);
    mBtnList.append(btnLogout);

    ToolButton* btnReboot = new ToolButton(this);
    btnReboot->setToolButtonStyle(Qt::ToolButtonTextOnly);
    btnReboot->setText(tr("Reboot"));//重启
    btnReboot->setFixedSize(POWERBTNSIZE);
    btnReboot->setGeometry(QRect(QPoint(0,0), POWERBTNSIZE));
    connect(btnReboot, &ToolButton::clicked, this, &PowerMenu::onComputerReboot);
    mLayout->addWidget(btnReboot);
    QPropertyAnimation * btnRebootAnim = new QPropertyAnimation(btnReboot, "pos", btnReboot);
    connect(btnRebootAnim, &QPropertyAnimation::finished, this, &PowerMenu::onAnimFinished);
    btnRebootAnim->setDuration(200);
    btnRebootAnim->setStartValue(QPoint(0,POWERBTNSIZE.height()*5));
    btnRebootAnim->setEndValue(QPoint(0,POWERBTNSIZE.height()*3));
    mBtns.insert( btnReboot, btnRebootAnim);
    mBtnList.append(btnReboot);

    ToolButton* btnPowerOff = new ToolButton(this);
    btnPowerOff->setToolButtonStyle(Qt::ToolButtonTextOnly);
    btnPowerOff->setText(tr("PowerOff"));//关机
    btnPowerOff->setFixedSize(POWERBTNSIZE);
    btnPowerOff->setGeometry(QRect(QPoint(0,0), POWERBTNSIZE));
    connect(btnPowerOff, &ToolButton::clicked, this, &PowerMenu::onComputerPowerOff);
    mLayout->addWidget(btnPowerOff);
    QPropertyAnimation * btnPowerOffAnim = new QPropertyAnimation(btnPowerOff, "pos", btnPowerOff);
    connect(btnPowerOffAnim, &QPropertyAnimation::finished, this, &PowerMenu::onAnimFinished);
    btnPowerOffAnim->setDuration(200);
    btnPowerOffAnim->setStartValue(QPoint(0,POWERBTNSIZE.height()*5));
    btnPowerOffAnim->setEndValue(QPoint(0,POWERBTNSIZE.height()*4));
    mBtns.insert( btnPowerOff, btnPowerOffAnim);
    mBtnList.append(btnPowerOff);

    //由于style中的HighlightedText颜色默认为黑色，这里设置其默认颜色为白色
    QPalette pa = btnSleep->palette();
    pa.setColor(QPalette::HighlightedText, QColor(Qt::white));
    btnSleep->setPalette(pa);
    btnSwitch->setPalette(pa);
    btnLogout->setPalette(pa);
    btnReboot->setPalette(pa);
    btnPowerOff->setPalette(pa);

    //隐藏不可用的功能，以下接口在v4版本中并不能生效
//    QDBusInterface * login1 = new QDBusInterface(QString::fromLatin1("org.freedesktop.login1"),
//                                                 QString::fromLatin1("/org/freedesktop/login1"),
//                                                 QString::fromLatin1("org.freedesktop.login1.Manager"),
//                                                 QDBusConnection::systemBus(), this);

//    QDBusMessage suspendMsg =  login1->call("CanReboot");
//    if(!suspendMsg.arguments().isEmpty())
//    {
//        if(suspendMsg.arguments().first().toString().contains("yes"))
//            btnReboot->show();
//        else
//            btnReboot->hide();
//    }
//    QDBusMessage poweroffMsg =  login1->call("CanPowerOff");
//    if(!poweroffMsg.arguments().isEmpty())
//    {
//        if(poweroffMsg.arguments().first().toString().contains("yes"))
//            btnPowerOff->show();
//        else
//            btnPowerOff->hide();
//    }
//    QDBusMessage hibrnateMsg =  login1->call("CanHibernate");
//    if(!hibrnateMsg.arguments().isEmpty())
//    {
//        if(hibrnateMsg.arguments().first().toString().contains("yes"))
//            btnSleep->show();
//        else
//            btnSleep->hide();
//    }

    if(checkVirtualMachine()) btnSleep->hide();
    if(checkZteVersion()) btnSwitch->hide();

    this->setLayout(mLayout);
}

void PowerMenu::expend(bool extend)
{
    //
    mExtend = extend;
    QHash<ToolButton*, QPropertyAnimation*>::iterator it = mBtns.begin();
    while (it != mBtns.end()) {
        QPropertyAnimation * anim = it.value();
        if(anim->state() == QPropertyAnimation::Running)
        {
            anim->pause();
        }
        anim->setDirection(extend ? QPropertyAnimation::Forward : QPropertyAnimation::Backward);
        anim->start();
        it++;
    }
}

QList<ToolButton *> PowerMenu::getBtns()
{
    return mBtnList;
}

void PowerMenu::onAnimFinished()
{
    QHash<ToolButton*, QPropertyAnimation*>::iterator it = mBtns.begin();
    while (it != mBtns.end()) {
        QPropertyAnimation * anim = it.value();
        if(anim->state() == QPropertyAnimation::Running)
        {
            return;
        }
        it++;
    }
    if(!mExtend) this->hide();

    signalAnimFinished();
}

void PowerMenu::onComputerSleep()
{
    QProcess::startDetached("nde-leave --hibernate");
    this->window()->hide();
}

void PowerMenu::onComputerSwitch()
{
    //nde-leave
    QProcess::startDetached("nde-leave");
    this->window()->hide();
}

void PowerMenu::onComputerLogout()
{
    //nde-leave --logout
    QProcess::startDetached("nde-leave --logout");
    this->window()->hide();
}

void PowerMenu::onComputerReboot()
{
    QProcess::startDetached("nde-leave --reboot");
    this->window()->hide();
}

void PowerMenu::onComputerPowerOff()
{
    QProcess::startDetached("nde-leave --shutdown");
    this->window()->hide();
}

bool PowerMenu::checkVirtualMachine()
{
    QProcess p;
    QStringList options;

    //检测是否是虚拟机vbox vmware，通过读取scsi文件中的设备判断虚拟机
    //    options << "-c" << "cat /proc/scsi/scsi | grep -i -e vbox -e vmware -e virtualbox";
    //读取cpuinfo，cpuinfo中包含hypervisor字段说明是虚拟机
    options << "-c" << "grep -q \"^flags.*\\ hypervisor\\ \" /proc/cpuinfo";
    p.start("/bin/bash", options);
    p.waitForFinished();
    p.waitForReadyRead();

    //exitCode=0 说明命令执行成功，是虚拟机;exitCode=1表示命令执行失败，是实体机
    int exitCode = p.exitCode();
    return exitCode ? false:true;
}

bool PowerMenu::checkZteVersion()
{
    QSettings setting("/etc/os-release", QSettings::IniFormat, this);
    QString version = setting.value("VERSION").toString();
    QString versionId = setting.value("VERSION_ID").toString();
    return (version.contains("zte", Qt::CaseInsensitive) || versionId.contains("zte", Qt::CaseInsensitive));
}
