#ifndef NOTIFYDISPLAYWIDGET_H
#define NOTIFYDISPLAYWIDGET_H

#include <QWidget>
#include <QMultiMap>
#include "notifygroup.h"
#include <QMutex>
#include <Nde/Settings>
#include "../panel/indepanelplugin.h"

namespace Ui {
class NotifyDisplayWidget;
}

class NotifyDisplayWidget : public QWidget
{
    Q_OBJECT

public:
    explicit NotifyDisplayWidget(INdePanelPlugin *plugin,QWidget *parent = nullptr);
    ~NotifyDisplayWidget();

    void realign();

signals:
    //通知中心失去焦点关闭窗口
    void lostFocus();
    //在通知中心用户点击关闭
    void notifyClosed(uint id, uint reason);
    //用户点击动作发出信号
    void actionTriggered(uint id, const QString &actionKey);

    void signalNotifiesCountChanged(int count);

public slots:
    //应用产生通知消息
    void onAddNotify(uint id, const QString &application,
                     const QString &summary, const QString &body,
                     const QString &icon, int timeout,
                     const QStringList& actions, const QVariantMap& hints,
                     bool noSave = false);
    //应用主动关闭通知消息
    void onRmoveNotify(uint id, uint reason);



protected:
    bool event(QEvent *event) override;
    virtual void resizeEvent(QResizeEvent *event) override;

private slots:
    void onGroupClose();
    void onClear();
    void onTimeout();
    //功能按钮
    void onSetting();
    void onRemote();
    void onScreenShot();
    void onMultiScreen();

private:
    Ui::NotifyDisplayWidget *m_ui;
    QVBoxLayout * m_notifyLayout;
    INdePanelPlugin * mPlugin;

    int mOpacity; //背景透明度
    QColor mBackgroundColor; //背景色

    QMutex mBuffMutex;
    QTimer * mTimer;
    QHash<uint, NotifyGroup*> m_groups;
    QList<NotifyGroup*> m_buffGroups;
};

#endif // NOTIFYDISPLAYWIDGET_H
