#ifndef NOTIFYGROUP_H
#define NOTIFYGROUP_H

#include <QWidget>
#include "notifyitem.h"
#include <QTimer>
#include "notifygrouptitle.h"

class NotifyGroup : public QWidget
{
    Q_OBJECT
    Q_PROPERTY(uint applicationID READ applicationID)
    Q_PROPERTY(int fixedHeight READ fixedHeight WRITE setFixedHeight)
public:
    explicit NotifyGroup(int width, uint id, QString name,
                         const QString &summary,
                         const QString &body,
                         const QString &icon,
                         const QStringList actions,
                         const QVariantMap& hints,
                         QWidget *parent = nullptr);

    inline uint applicationID() const {return m_appID;}

    void addNotifyItem(const QString name,
                       const QString &summary,
                       const QString &body,
                       const QString &icon,
                       const QStringList actions,
                       const QVariantMap& hints);
    bool animationFinished() const;

    //property
    inline int fixedHeight(){return this->height();}

signals:
    void signalGroupClose(uint id);
    void actionTriggered(uint id, const QString &actionKey);

public slots:
    void onItemExpend(bool extend);
    void onItemClose();
    void onGroupShow();
    void onGroupClose();
    void onGroupExtend(bool extend);

protected:
    virtual void resizeEvent(QResizeEvent *event);

private slots:
    void slotAnimationCloseFinished();
    void slotAnimationShowFinished();
    void slotAnimationExtendFinished();
    void slotTimeout();

private:
    int calculateHeight(bool extend);

private:
    bool m_extend;
    uint m_appID;
    QList<NotifyItem*> m_items;

    //界面元素
    NotifyGroupTitle * m_title;
    QVBoxLayout * m_layout;
    QWidget * m_maskWgt;

    //动画
    QPropertyAnimation * m_animationClose;
    QPropertyAnimation * m_animationShow;
    QPropertyAnimation * m_animationExtend;
//    QPropertyAnimation * m_animationChildExtend;

    //缓冲机制
    QList<NotifyItem*> m_buffItems;
    QTimer * mTimer;
};

#endif // NOTIFYGROUP_H

