#include "notifygrouptitle.h"
#include "ui_notifygrouptitle.h"
#include <QDBusArgument>
#include <QFile>
#include <QUrl>
#include <XdgIcon>
#include <KWindowEffects>
#include <QStyleOption>
#include <QPainter>
#include <QDebug>
#define ICONSIZE QSize(32, 32)

NotifyGroupTitle::NotifyGroupTitle(int width,
                                   uint id,
                                   QString name,
                                   const QString &summary,
                                   const QString &body,
                                   const QString &icon,
                                   const QStringList actions,
                                   const QVariantMap &hints,
                                   QWidget *parent) :
    QWidget(parent),
    ui(new Ui::NotifyGroupTitle),
    m_appID(id),
    m_appName(name),
    m_extend(false)
{
    ui->setupUi(this);

    this->setObjectName("NotifyGroupTitle");
    this->setFocusPolicy(Qt::NoFocus);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);

//    KWindowEffects::enableBlurBehind(this->winId(), true);

    QFont f = ui->label_application->font();
    f.setBold(true);
    f.setPixelSize(14);
    ui->label_application->setFont(f);
    QFontMetrics fontMetrics = QFontMetrics(ui->label_application->font());
    QString elideName = fontMetrics.elidedText(name, Qt::ElideRight, ui->label_application->width());
    ui->label_application->setText(elideName);


    f = ui->label_notice->font();
    f.setPixelSize(14);
    ui->label_notice->setFont(f);

    if (!hints[QLatin1String("image_data")].isNull())
    {
        m_pixmap = getPixmapFromHint(hints[QLatin1String("image_data")]);
    }
    else if (!hints[QLatin1String("image_path")].isNull())
    {
        m_pixmap = getPixmapFromString(hints[QLatin1String("image_path")].toString());
    }
    else if (!icon.isEmpty())
    {
        m_pixmap = getPixmapFromString(icon);
    }
    else if (!hints[QLatin1String("icon_data")].isNull())
    {
       m_pixmap = getPixmapFromHint(hints[QLatin1String("icon_data")]);
    }
    if (m_pixmap.isNull())
    {
        ui->label_icon->hide();
    }
    else
    {
        if (m_pixmap.size().width() > ICONSIZE.width()
            || m_pixmap.size().height() > ICONSIZE.height())
        {
            m_pixmap = m_pixmap.scaled(ICONSIZE, Qt::KeepAspectRatio, Qt::SmoothTransformation);
        }
        ui->label_icon->setPixmap(m_pixmap);
        ui->label_icon->show();
    }

    setNotifyCount(0);

    this->setFixedSize(width, this->sizeHint().height());

    ui->btn_close->hide();
    connect(ui->btn_close, &QToolButton::clicked, this, &NotifyGroupTitle::signalGroupClose);
}

NotifyGroupTitle::~NotifyGroupTitle()
{
    delete ui;
}

void NotifyGroupTitle::setNotifyCount(int count)
{
    ui->label_notice->setText(tr("%1 notifies contained.").arg(count));
}

void NotifyGroupTitle::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void NotifyGroupTitle::mousePressEvent(QMouseEvent *event)
{
    QWidget::mousePressEvent(event);
}

void NotifyGroupTitle::mouseReleaseEvent(QMouseEvent *event)
{
    m_extend = !m_extend;
    emit signalGroupExtend(m_extend);
    QWidget::mouseReleaseEvent(event);
}

void NotifyGroupTitle::enterEvent(QEvent *event)
{
    ui->btn_close->show();
    QWidget::enterEvent(event);
}

void NotifyGroupTitle::leaveEvent(QEvent *event)
{
    ui->btn_close->hide();
    QWidget::leaveEvent(event);
}

QPixmap NotifyGroupTitle::getPixmapFromHint(const QVariant &argument) const
{
    int width, height, rowstride, bitsPerSample, channels;
    bool hasAlpha;
    QByteArray data;

    const QDBusArgument arg = argument.value<QDBusArgument>();
    arg.beginStructure();
    arg >> width;
    arg >> height;
    arg >> rowstride;
    arg >> hasAlpha;
    arg >> bitsPerSample;
    arg >> channels;
    arg >> data;
    arg.endStructure();

    bool rgb = !hasAlpha && channels == 3 && bitsPerSample == 8;
    QImage::Format imageFormat = rgb ? QImage::Format_RGB888 : QImage::Format_ARGB32;

    QImage img = QImage((uchar*)data.constData(), width, height, imageFormat);

    if (!rgb)
        img = img.rgbSwapped();

    return QPixmap::fromImage(img);
}

QPixmap NotifyGroupTitle::getPixmapFromString(const QString &str) const
{
    QUrl url(str);
    if (url.isValid() && QFile::exists(url.toLocalFile()))
    {
        return QPixmap(url.toLocalFile());
    }
    else
    {
        return XdgIcon::fromTheme(str/*, XdgIcon::defaultApplicationIcon()*/).pixmap(ICONSIZE);
    }
}

