#include "notifyitem.h"
#include <QVariantMap>
#include <QDBusMessage>
#include <QDBusArgument>
#include <QUrl>
#include <QFile>
#include <XdgIcon>
#include <QIcon>
#include <QDebug>
#include <KWindowEffects>
#include <QStyleOption>
#include <QPaintEvent>
#include <QPainter>

#define ICONSIZE QSize(32, 32)

NotifyItem::NotifyItem(const int itemWidth,
        const uint appId,
        const QString &application,
        const QString &summary,
        const QString &body,
        const QString &icon,
        const QString action,
        const QVariantMap &hints,
        QWidget *parent)
    : QWidget(parent)
{
    this->setObjectName("NotifyItem");
    this->setFocusPolicy(Qt::NoFocus);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);

    //    KWindowEffects::enableBlurBehind(this->winId(), true);

    //main layout
    m_layout = new QHBoxLayout;
    m_layout->setContentsMargins(12,16,12,16);
    m_layout->setSpacing(10);

    //icon layout
    m_labelIcon = new QLabel(this);
    m_labelIcon->setObjectName("label_icon");
    m_labelIcon->setFixedSize(32,32);

    m_layout_icon = new QVBoxLayout;
    m_layout_icon->setMargin(0);
    m_layout_icon->setContentsMargins(0,0,0,0);
    m_layout_icon->setSpacing(0);
    m_layout_icon->addWidget(m_labelIcon);
    m_layout_icon->addStretch();

    //btn layout
    m_btn_close = new QToolButton(this);
    m_btn_close->setObjectName("btn_close");
    m_btn_close->setFixedSize(28,28);
    m_btn_close->setStyleSheet(getBtnCloseStyleSheet());
    m_btn_close->setIconSize(QSize(28, 28));
    m_btn_close->hide();

    m_btn_extend = new QToolButton(this);
    m_btn_extend->setObjectName("btn_extend");
    m_btn_extend->setFixedSize(28,28);
    m_btn_extend->setCheckable(true);
    m_btn_extend->setIconSize(QSize(28, 28));
    m_btn_extend->setStyleSheet(getBtnExtendStyleSheet());
    m_btn_extend->hide();

    m_layout_btn = new QVBoxLayout;
    m_layout_btn->setMargin(0);
    m_layout_btn->setSpacing(0);
    m_layout_btn->setContentsMargins(0,0,0,0);
    m_layout_btn->addWidget(m_btn_close);
    m_layout_btn->addWidget(m_btn_extend);
    m_layout_btn->addStretch(0);


    //计算文字label的宽度
    int width = itemWidth - m_layout->contentsMargins().left() -
            m_layout->contentsMargins().right() - m_layout->spacing()*2 -
            m_labelIcon->width() - m_btn_close->width();

    //text layout
    m_labelApplication = new QLabel(this);
    m_labelApplication->setObjectName("label_application");
    m_labelApplication->setFixedWidth(width);
    QFont f = m_labelApplication->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelApplication->setFont(f);
    m_labelApplication->setTextFormat(Qt::PlainText);

    m_labelSummary = new QLabel(this);
    m_labelSummary->setObjectName("label_summary");
    m_labelSummary->setFixedWidth(width);
    f = m_labelSummary->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelSummary->setFont(f);
    m_labelSummary->setTextFormat(Qt::RichText);
    m_labelSummary->setWordWrap(true);

    m_labelBody = new QLabel(this);
    m_labelBody->setObjectName("label_body");
    m_labelBody->setFixedWidth(width);
    f = m_labelBody->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelBody->setFont(f);
    m_labelBody->setTextFormat(Qt::RichText);
    m_labelBody->setWordWrap(true);
    m_labelBody->setOpenExternalLinks(true);
    m_labelBody->hide();

    m_layout_text = new QVBoxLayout;
    m_layout_text->setMargin(0);
    m_layout_text->setSpacing(5);
    m_layout_text->setContentsMargins(0,0,0,0);
    m_layout_text->addWidget(m_labelApplication);
    m_layout_text->addWidget(m_labelSummary);
    m_layout_text->addWidget(m_labelBody);
    m_layout_text->addStretch(0);

    m_layout->addLayout(m_layout_icon);
    m_layout->addLayout(m_layout_text);
    m_layout->addLayout(m_layout_btn);
    m_layout->addStretch(0);

    setValues(appId, application, summary, body, icon, action, hints);

    //这两个label_icon，label_application 不用安装事件过滤器，widget会接收到其mouse事件，原因待查
//    m_ui->label_icon->installEventFilter(this);
//    m_ui->label_application->installEventFilter(this);
    m_labelSummary->installEventFilter(this);
    m_labelBody->installEventFilter(this);

    //animation
    m_animationClose = new QPropertyAnimation(this, "geometry", this);
    m_animationClose->setDuration(200);
    m_animationClose->setEasingCurve(QEasingCurve::InOutCubic);
    connect(m_animationClose, &QPropertyAnimation::finished, this, &NotifyItem::slotAnimationCloseFinished);

    m_animationShow = new QPropertyAnimation(this, "geometry", this);
    m_animationShow->setDuration(200);
    m_animationShow->setEasingCurve(QEasingCurve::InOutCubic);
    connect(m_animationShow, &QPropertyAnimation::finished, this, &NotifyItem::slotAnimationShowFinished);

    m_animationExtend = new QPropertyAnimation(this, "fixedHeight", this);
    m_animationExtend->setDuration(200);
    connect(m_animationExtend, &QPropertyAnimation::finished, this, &NotifyItem::slotAnimationExtendFinished);

    m_animationMove = new QPropertyAnimation(this, "pos", this);
    m_animationMove->setDuration(200);
    connect(m_animationMove, &QPropertyAnimation::finished, this, &NotifyItem::slotAnimationMoveFinished);

    connect(m_btn_close, &QToolButton::clicked, this, &NotifyItem::slotClose);
    connect(m_btn_extend, &QToolButton::clicked,this,&NotifyItem::slotExtend);

    this->setFixedSize(itemWidth,getHeight(false));
    this->setLayout(m_layout);
}

void NotifyItem::setValues(const uint appId,
        const QString &application,
        const QString &summary,
        const QString &body,
        const QString &icon,
        const QString action,
        const QVariantMap &hints)
{
    m_appId = appId;

    if (!hints[QLatin1String("image_data")].isNull())
    {
        m_pixmap = getPixmapFromHint(hints[QLatin1String("image_data")]);
    }
    else if (!hints[QLatin1String("image_path")].isNull())
    {
        m_pixmap = getPixmapFromString(hints[QLatin1String("image_path")].toString());
    }
    else if (!icon.isEmpty())
    {
        m_pixmap = getPixmapFromString(icon);
    }
    else if (!hints[QLatin1String("icon_data")].isNull())
    {
       m_pixmap = getPixmapFromHint(hints[QLatin1String("icon_data")]);
    }
    // issue #325: Do not display icon if it's not found...
    if (m_pixmap.isNull())
    {
//        m_labelIcon->hide();
    }
    else
    {
        if (m_pixmap.size().width() > ICONSIZE.width()
            || m_pixmap.size().height() > ICONSIZE.height())
        {
            m_pixmap = m_pixmap.scaled(ICONSIZE, Qt::KeepAspectRatio, Qt::SmoothTransformation);
        }
        m_labelIcon->setPixmap(m_pixmap);
        m_labelIcon->show();
    }
    //XXX: workaround to properly set text labels widths (for correct sizeHints after setText)

    // application
//    m_ui->label_application->setVisible(!application.isEmpty());
//    m_ui->label_appName->setFixedWidth(m_ui->label_application->width());
    QFontMetrics fontMetrics = QFontMetrics(m_labelApplication->font());
    QString elideApplication = fontMetrics.elidedText(application, Qt::ElideRight, m_labelApplication->width());
    m_labelApplication->setText(elideApplication);

    // summary
//    m_ui->label_summary->setVisible(!summary.isEmpty() && application != summary);
//    m_ui->label_summary->setFixedWidth(m_ui->label_summary->width());
    m_labelSummary->setText(summary);
    QFontMetrics fontMetrics1 = QFontMetrics(m_labelSummary->font());
    QString elideSummary = fontMetrics1.elidedText(summary, Qt::ElideRight, m_labelSummary->width());
    m_labelSummary->setText(elideSummary);

    // body
//    m_ui->label_body->setVisible(!body.isEmpty());
//    m_ui->label_body->setFixedWidth(m_ui->label_body->width());
    //https://developer.gnome.org/notification-spec
    //Body - This is a multi-line body of text. Each line is a paragraph, server implementations are free to word wrap them as they see fit.
    //XXX: remove all unsupported tags?!? (supported <b>, <i>, <u>, <a>, <img>)
    QString formatted(body);
    m_labelBody->setText(formatted.replace(QL1C('\n'), QStringLiteral("<br/>")));

    // Categories *********************
    if (!hints[QLatin1String("category")].isNull())
    {
        // TODO/FIXME: Categories - how to handle it?
    }

    // Urgency Levels *********************
    // Type    Description
    // 0   Low
    // 1   Normal
    // 2   Critical
    if (!hints[QLatin1String("urgency")].isNull())
    {
        // TODO/FIXME: Urgencies - how to handle it?
    }

    m_action = action;

    // Actions
//    if (actions.count() && m_actionWidget == 0)
//    {
//        if (actions.count()/2 < 4)
//            m_actionWidget = new NotificationActionsButtonsWidget(actions, this);
//        else
//            m_actionWidget = new NotificationActionsComboWidget(actions, this);

//        connect(m_actionWidget, &NotificationActionsWidget::actionTriggered,
//                this, &NotifyItem::actionTriggered);

//        m_ui->layout_Text->addWidget(m_actionWidget);
//        m_actionWidget->show();
//    }

//    adjustSize();
    // ensure layout expansion
    //    setMinimumHeight(qMax(rect().height(), childrenRect().height()));
}

bool NotifyItem::animationFinished()
{
    return m_animationShow->state() != QPropertyAnimation::Running &&
            m_animationClose->state() != QPropertyAnimation::Running &&
            m_animationExtend->state() != QPropertyAnimation::Running;
}

int NotifyItem::getHeight()
{
    bool extend = m_btn_extend->isChecked();
    return getHeight(extend);
}

void NotifyItem::slotShow()
{
    if(m_animationShow->state() == QPropertyAnimation::Running)
    {
        return;
    }
    m_animationShow->setStartValue(this->geometry().adjusted(-this->width(),0,-this->width(),0));
    m_animationShow->setEndValue(this->geometry());
    m_animationShow->start();
}

void NotifyItem::slotClose()
{
    if(m_animationClose->state() == QPropertyAnimation::Running)
    {
        return;
    }
    m_animationClose->setStartValue(this->geometry());
    m_animationClose->setEndValue(this->geometry().adjusted(-this->width(),0,-this->width(),0));
    m_animationClose->start();
}

void NotifyItem::slotExtend()
{
    ////extend
    bool extend = m_btn_extend->isChecked();

    m_btn_extend->setStyleSheet(getBtnExtendStyleSheet());

    if(m_animationExtend->state() == QPropertyAnimation::Running)
    {
        m_animationExtend->pause();
    }
    m_animationExtend->setStartValue(getHeight(!extend));
    m_animationExtend->setEndValue(getHeight(extend));
    m_animationExtend->start();

    if(extend)
    {
        m_labelBody->setVisible(extend);
        signalExpend(true);
    }
}

void NotifyItem::slotMove(bool moveDown, QPoint startP, QPoint endP)
{
    this->setVisible(true);

    if(m_animationMove->state() == QPropertyAnimation::Running)
    {
        m_animationMove->pause();
//        m_animationMove->setDirection(moveDown ? QPropertyAnimation::Forward : QPropertyAnimation::Backward);
//        m_animationMove->resume();
    }

    m_animationMove->setStartValue(startP);
    m_animationMove->setEndValue(endP);
    m_animationMove->setDirection(moveDown ? QPropertyAnimation::Forward : QPropertyAnimation::Backward);
    m_animationMove->start();
}

void NotifyItem::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void NotifyItem::mousePressEvent(QMouseEvent *event)
{
    QWidget::mousePressEvent(event);
}

void NotifyItem::mouseReleaseEvent(QMouseEvent *event)
{
    //
    if(!m_action.isEmpty())
    {
        emit actionTriggered(m_appId, m_action);
    }

    QWidget::mouseReleaseEvent(event);
}

void NotifyItem::enterEvent(QEvent *event)
{
    m_btn_close->show();
    m_btn_extend->show();
    QWidget::enterEvent(event);
}

void NotifyItem::leaveEvent(QEvent *event)
{
    m_btn_close->hide();
    m_btn_extend->hide();
    QWidget::leaveEvent(event);
}

bool NotifyItem::eventFilter(QObject *watched, QEvent *event)
{
    if(watched == m_labelIcon ||
            watched == m_labelApplication ||
            watched == m_labelSummary ||
            watched == m_labelBody)
    {
        if(event->type() == QEvent::MouseButtonRelease )
        {
            if(!m_action.isEmpty()) emit actionTriggered(m_appId, m_action);
        }
    }
    return QWidget::eventFilter(watched, event);
}

void NotifyItem::slotAnimationCloseFinished()
{
    QWidget * w = this->parentWidget();
    if(w && w->layout())
    {
        w->layout()->removeWidget(this);
    }
    this->hide();
    this->deleteLater();

    emit signalClose();
}
void NotifyItem::slotAnimationShowFinished()
{
    this->parentWidget()->update();
}

void NotifyItem::slotAnimationExtendFinished()
{
    bool extend = m_btn_extend->isChecked();
    if(!extend)
    {
        m_labelBody->setVisible(false);
        signalExpend(false);
    }
}

void NotifyItem::slotAnimationMoveFinished()
{
    bool moveDown = (m_animationMove->direction() == QPropertyAnimation::Forward);
    this->setVisible(moveDown);
}

QPixmap NotifyItem::getPixmapFromHint(const QVariant &argument) const
{
    int width, height, rowstride, bitsPerSample, channels;
    bool hasAlpha;
    QByteArray data;

    const QDBusArgument arg = argument.value<QDBusArgument>();
    arg.beginStructure();
    arg >> width;
    arg >> height;
    arg >> rowstride;
    arg >> hasAlpha;
    arg >> bitsPerSample;
    arg >> channels;
    arg >> data;
    arg.endStructure();

    bool rgb = !hasAlpha && channels == 3 && bitsPerSample == 8;
    QImage::Format imageFormat = rgb ? QImage::Format_RGB888 : QImage::Format_ARGB32;

    QImage img = QImage((uchar*)data.constData(), width, height, imageFormat);

    if (!rgb)
        img = img.rgbSwapped();

    return QPixmap::fromImage(img);
}

QPixmap NotifyItem::getPixmapFromString(const QString &str) const
{
    QUrl url(str);
    if (url.isValid() && QFile::exists(url.toLocalFile()))
    {
        return QPixmap(url.toLocalFile());
    }
    else
    {
        return XdgIcon::fromTheme(str/*, XdgIcon::defaultApplicationIcon()*/).pixmap(ICONSIZE);
    }
}

int NotifyItem::getHeight(bool extend)
{
    int applicationHeight = m_labelApplication->heightForWidth(m_labelApplication->width());
    if(applicationHeight <= 0) applicationHeight = 22;
    int summaryHeight = m_labelSummary->heightForWidth(m_labelSummary->width());
    if(summaryHeight <= 0) summaryHeight = 22;
    int bodyHeight = m_labelBody->heightForWidth(m_labelBody->width());
    if(bodyHeight <= 0) bodyHeight = 22;

    int height =  m_layout->contentsMargins().top() +
            applicationHeight +
            m_layout_text->spacing() +
            summaryHeight +
            m_layout->contentsMargins().bottom();

    height += extend ? bodyHeight+m_layout_text->spacing():0;

    return height;
}

QString NotifyItem::getBtnExtendStyleSheet()
{
    QString btnExtendStyleSheet = QString("QToolButton#btn_extend{"
                                          "image:url(:/res/%1-normal.png);"
                                          "background-color: none;"
                                          "}"
                                          "QToolButton#btn_extend:hover{"
                                          "image:url(:/res/%2-hover.png);"
                                          "background-color: none;"
                                          "}"
                                          "QToolButton#btn_extend:pressed{"
                                          "image:url(:/res/%3-click.png);"
                                          "background-color: none;"
                                          "}"
                                          );

    if(m_btn_extend->isChecked())
    {
        return btnExtendStyleSheet.arg("pull").arg("pull").arg("pull");
    }
    return btnExtendStyleSheet.arg("extend").arg("extend").arg("extend");
}

QString NotifyItem::getBtnCloseStyleSheet()
{
    QString btnCloseStyleSheet = QString("QToolButton#btn_close{"
                                          "image:url(:/res/close-normal.png);"
                                          "background-color: none;"
                                          "}"
                                          "QToolButton#btn_close:hover{"
                                          "image:url(:/res/close-hover.png);"
                                          "background-color: none;"
                                          "}"
                                          "QToolButton#btn_close:pressed{"
                                          "image:url(:/res/close-click.png);"
                                          "background-color: none;"
                                          "}"
                                          );
    return btnCloseStyleSheet;
}
