#ifndef NOTIFYITEM_H
#define NOTIFYITEM_H

#include <QWidget>
#include <QPixmap>
#include <QPropertyAnimation>
#include <QLabel>
#include <QHBoxLayout>
#include <QToolButton>

#define NOTIFICATIONBAR_WIDTH 320

class NotifyItem : public QWidget
{
    Q_OBJECT
    Q_PROPERTY(uint applicationID READ applicationID )
    Q_PROPERTY(int fixedHeight READ fixedHeight WRITE setFixedHeight)
public:
    explicit NotifyItem(const int itemWidth,
                        const uint appId,
                        const QString &application,
                        const QString &summary,
                        const QString &body,
                        const QString &icon,
                        const QString action,
                        const QVariantMap& hints,

                        QWidget *parent = nullptr);

    //
    void setValues(const uint appId,
                   const QString &application,
                   const QString &summary,
                   const QString &body,
                   const QString &icon,
                   const QString action,
                   const QVariantMap& hints);

    uint applicationID() const {return m_appId;}
    //property
    inline int fixedHeight(){return this->height();}

    bool animationFinished();

    int getHeight() ;

signals:
    void signalExpend(bool expend);
    void signalClose();
    void actionTriggered(uint id, const QString &actionKey);

public slots:
    void slotShow();
    void slotClose();
    void slotExtend();
    void slotMove(bool moveDown, QPoint startP, QPoint endP);

protected:
    virtual void paintEvent(QPaintEvent *event);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);
    virtual void enterEvent(QEvent *event);
    virtual void leaveEvent(QEvent *event);
    virtual bool eventFilter(QObject *watched, QEvent *event);

private slots:
    void slotAnimationCloseFinished();
    void slotAnimationShowFinished();
    void slotAnimationExtendFinished();
    void slotAnimationMoveFinished();

private:
    QPixmap getPixmapFromHint(const QVariant &argument) const;
    QPixmap getPixmapFromString(const QString &str) const;
    int getHeight( bool extend ) ;
    QString getBtnExtendStyleSheet();
    QString getBtnCloseStyleSheet();

private:
    uint m_appId;

    //界面元素
    QPixmap m_pixmap; //tubiaowenjian
    QHBoxLayout * m_layout;
    QLabel * m_labelIcon;
    QVBoxLayout * m_layout_icon;
    QLabel * m_labelApplication;
    QLabel * m_labelSummary;
    QLabel * m_labelBody;
    QVBoxLayout * m_layout_text;
    QToolButton * m_btn_close;
    QToolButton * m_btn_extend;
    QVBoxLayout * m_layout_btn;

    QPropertyAnimation * m_animationClose;
    QPropertyAnimation * m_animationShow;
    QPropertyAnimation * m_animationExtend;
    QPropertyAnimation * m_animationMove;
    QString m_action;

};

#endif // NOTIFYITEM_H
