/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2010-2011 Razor team
 * Authors:
 *   Petr Vanek <petr@scribus.info>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <QAction>
#include <QX11Info>
#include <nde-globalkeys.h>
#include <XdgIcon>
#include <Nde/Notification>
#include <KWindowSystem/KWindowSystem>
#include <KWindowSystem/NETWM>
#include "showdesktop.h"
#include "../panel/pluginsettings.h"
#include "QDebug"

#define DEFAULT_SHORTCUT "Control+Alt+D"
#define DEFAULT_SHORTCUT_WIN "Meta+D"
#define DEFAULT_SHORTCUT_MINIMIZE_WIN "Meta+M"

ShowDesktop::ShowDesktop(const INdePanelPluginStartupInfo &startupInfo) :
    QObject(),
    INdePanelPlugin(startupInfo)
{
    m_key = GlobalKeyShortcut::Client::instance()->addAction(QString(), QString("/panel/%1/show_hide").arg(settings()->group()), tr("Show desktop"), this);
    if (m_key)
    {
        connect(m_key, &GlobalKeyShortcut::Action::registrationFinished, this, &ShowDesktop::shortcutRegistered);
        connect(m_key, SIGNAL(activated()), this, SLOT(toggleShowingDesktop()));
    }

    m_show_desktop = GlobalKeyShortcut::Client::instance()->addAction(QString(), QString("/panel/%1/show_desktop").arg(settings()->group()), tr("Show desktop"), this);
    if(m_show_desktop)
    {
        connect(m_show_desktop, &GlobalKeyShortcut::Action::registrationFinished, [this] {
            if(m_show_desktop->shortcut().isEmpty())
                m_show_desktop->changeShortcut(DEFAULT_SHORTCUT_WIN);
        });
        connect(m_show_desktop, SIGNAL(activated()), this, SLOT(toggleShowingDesktop()));
    }

    // minimize all windows
    m_minimize_win = GlobalKeyShortcut::Client::instance()->addAction(QString(), QString("/panel/%1/minimize_win").arg(settings()->group()), tr("minimize all windows"), this);
    if(m_minimize_win)
    {
        connect(m_minimize_win, &GlobalKeyShortcut::Action::registrationFinished, [this] {
            if(m_minimize_win->shortcut().isEmpty())
                m_minimize_win->changeShortcut(DEFAULT_SHORTCUT_MINIMIZE_WIN);
        });
        connect(m_minimize_win, SIGNAL(activated()), this, SLOT(minimizeAllWindows()));
    }

    mTaskBarWidth = startupInfo.ndePanel->teskbarWidth();

    QAction * act = new QAction(this);
    connect(act, SIGNAL(triggered()), this, SLOT(toggleShowingDesktop()));


    mButton.setDefaultAction(act);
    mButton.setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    mButton.setAutoRaise(true);
}

void ShowDesktop::realign()
{
    if(this->panel()->position() > 1){
        mButton.setStyleSheet("border-top:1px solid rgba(255,255,255,0.2);");
        mButton.setFixedHeight(8);
        mButton.setFixedWidth(mTaskBarWidth);

    }else {
        mButton.setStyleSheet("border-left:1px solid rgba(255,255,255,0.2);");
        mButton.setFixedWidth(8);
        mButton.setFixedHeight(mTaskBarWidth);
    }
}

void ShowDesktop::shortcutRegistered()
{
    if (m_key->shortcut().isEmpty())
    {
        m_key->changeShortcut(DEFAULT_SHORTCUT);
        if (m_key->shortcut().isEmpty())
        {
            Nde::Notification::notify(tr("Show Desktop: Global shortcut '%1' cannot be registered").arg(DEFAULT_SHORTCUT));
        }
    }
}

void ShowDesktop::toggleShowingDesktop()
{
    KWindowSystem::setShowingDesktop(!KWindowSystem::showingDesktop());
}

void ShowDesktop::minimizeAllWindows()
{
    const auto wnds = KWindowSystem::stackingOrder();
    for (auto const wnd: wnds)
    {
        KWindowSystem::minimizeWindow(wnd);
    }
}
#undef DEFAULT_SHORTCUT
