#include "statusnotifierpopup.h"
#include <QDragEnterEvent>
#include <QMimeData>
#include <QDebug>
#include <QtMath>
#include <KWindowEffects>
#include "../panel/indepanelplugin.h"
StatusNotifierPopup::StatusNotifierPopup(INdePanelPlugin *plugin, QWidget *parent)
    : StatusNotifierWidget(plugin, "popupOrder", parent),
      mPlugin(plugin),
      mOpacity(-1)
{
    setObjectName("StatusNotifierPopup");
    setAcceptDrops(true);
    setWindowFlags(Qt::FramelessWindowHint  | Qt::X11BypassWindowManagerHint | Qt::WindowStaysOnTopHint);
    setAttribute(Qt::WA_TranslucentBackground);
    setAttribute(Qt::WA_NoSystemBackground, false);
    KWindowEffects::enableBlurBehind(winId(), true);
}

void StatusNotifierPopup::realign()
{
    Nde::GridLayout *layout = qobject_cast<Nde::GridLayout*>(this->layout());
    int columnCount = qCeil(qSqrt(layout->count()));
    columnCount = columnCount < 1 ? 1 : columnCount;
    int rowCount = qCeil(layout->count() / (qreal)columnCount);
    rowCount = rowCount < 1 ? 1 : rowCount;
    layout->setEnabled(false);
    layout->setColumnCount(columnCount);
    layout->setRowCount(rowCount);

    int width = 42 * columnCount;
    int height = 42 * rowCount;
    resize(width, height);
    layout->setEnabled(true);

    int opacity = mPlugin->panel()->opacity();
    QColor bColor = mPlugin->panel()->backgroundColor();

    if(mOpacity == opacity && mBackgroundColor == bColor )
        return;

    mOpacity = opacity;
    mBackgroundColor = bColor;

    QString color = QString("%1, %2, %3, %4")
        .arg(mBackgroundColor.red())
        .arg(mBackgroundColor.green())
        .arg(mBackgroundColor.blue())
        .arg(mOpacity / 100.0);
    setStyleSheet(QString("#StatusNotifierPopup { "
                          "background-color: rgba(" + color + "); }"));
}

bool StatusNotifierPopup::event(QEvent *event)
{
    if(this->geometry().contains(QCursor::pos()))
        return QWidget::event(event);
    if(event->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            this->hide();
        }
    }
    return QWidget::event(event);
}
