﻿/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2011 Razor team
 *            2014 Nde team
 * Authors:
 *   Alexander Sokoloff <sokoloff.a@gmail.com>
 *   Maciej Płaza <plaza.maciej@gmail.com>
 *   Kuzma Shapran <kuzma.shapran@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <QApplication>
#include <QDebug>
#include <QSignalMapper>
#include <QToolButton>
#include <QSettings>
#include <QList>
#include <QMimeData>
#include <QWheelEvent>
#include <QFlag>
#include <QX11Info>
#include <QTimer>
#include <QFileInfo>
#include <XdgDesktopFile>
#include <nde-globalkeys.h>
#include <Nde/GridLayout>
#include <XdgIcon>
#include <XdgMenu>
#include "ndetaskbar.h"
#include "ndetaskgroup.h"
#include <QProcess>
#include "../panel/thumbnailmanager.h"

using namespace Nde;

/************************************************

************************************************/
NdeTaskBar::NdeTaskBar(INdePanelPlugin *plugin, QWidget *parent) :
    QFrame(parent),
    mSignalMapper(new QSignalMapper(this)),
    mButtonStyle(Qt::ToolButtonIconOnly),
    mButtonWidth(48),
    mButtonMinWidth(48),
    mButtonHeight(42),
    mCloseOnMiddleClick(true),
    mRaiseOnCurrentDesktop(true),
    mShowOnlyOneDesktopTasks(false),
    mShowDesktopNum(0),
    mShowOnlyCurrentScreenTasks(true),
    mShowOnlyMinimizedTasks(false),
    mAutoRotate(true),
    mGroupingEnabled(true),
    mShowGroupOnHover(true),
    mIconByClass(false),
    mCycleOnWheelScroll(true),
    mPlugin(plugin),
    mStyle(new LeftAlignedTextStyle())
{
    ///初始化翻页按钮
    ///
    mCountPage = 1;
    mCurrentPage = 0;
    mCurrentPosition = mPlugin->panel()->position();
    //翻页按钮相关
    int size = getPanelSize();
    mPrviBtn = new StatusNotifierArrow(this);
    mPrviBtn->setObjectName("TaskBar_prvi_btn");
    mPrviBtn->setFixedSize(QSize(size/2-4,size/2-4));
    mPrviBtn->setToolButtonStyle(Qt::ToolButtonIconOnly);
    mPrviBtn->setOrigin(Qt::TopLeftCorner);
    connect(mPrviBtn, &QToolButton::clicked,this,&NdeTaskBar::slotPrviBtnClicked);
    mNextBtn = new StatusNotifierArrow(this);
    mNextBtn->setObjectName("TaskBar_next_btn");
    mNextBtn->setFixedSize(QSize(size/2-4,size/2-4));
    mNextBtn->setToolButtonStyle(Qt::ToolButtonIconOnly);
    mPrviBtn->setOrigin(Qt::TopLeftCorner);
    connect(mNextBtn, &QToolButton::clicked,this,&NdeTaskBar::slotNextBtnClicked);

    //翻页按钮Layout
    mPageLayout = new QBoxLayout(QBoxLayout::TopToBottom);
    mPageLayout->setSpacing(0);
    mPageLayout->setContentsMargins(0,0,0,0);
    mPageLayout->setMargin(2);
    mPageLayout->addWidget(mPrviBtn);
    mPageLayout->addWidget(mNextBtn);
    //翻页按钮widget
    mPageWidget = new QWidget(this);
    mPageWidget->setFixedSize(size/2, size);
    mPageWidget->setObjectName("PageWidget");
    mPageWidget->setMinimumSize(QSize(5,5));
    mPageWidget->setLayout(mPageLayout);

    //应用按钮布局
    setStyle(mStyle);
    mTasksLayout = new Nde::GridLayout();
    setLayout(mTasksLayout);
    mTasksLayout->setMargin(0);
    mTasksLayout->setStretch(Nde::GridLayout::StretchHorizontal | Nde::GridLayout::StretchVertical);
    //应用按钮widget
    mTasksWidget = new QWidget(this);
    mTasksWidget->setObjectName("TasksWidget");
    mTasksWidget->setMinimumSize(QSize(5,5));
    mTasksWidget->setLayout(mTasksLayout);
    mTasksWidget->setAcceptDrops(true);
    mTasksWidget->installEventFilter(this);

    //主布局
    mLayout = new QBoxLayout(QBoxLayout::LeftToRight);
    mLayout->setMargin(0);
    mLayout->setSpacing(0);
    mLayout->addWidget(mTasksWidget);
    mLayout->addStretch();
    mLayout->addWidget(mPageWidget);

    setLayout(mLayout);

    realign();

    QTimer::singleShot(0, this, SLOT(settingsChanged()));
    setAcceptDrops(true);

    connect(mSignalMapper, static_cast<void (QSignalMapper::*)(int)>(&QSignalMapper::mapped), this, &NdeTaskBar::activateTask);
    QTimer::singleShot(0, this, &NdeTaskBar::registerShortcuts);
    connect(ThumbnailManager::getIns(), &ThumbnailManager::windowAdded, this, &NdeTaskBar::onWindowAdded);
    connect(ThumbnailManager::getIns(), &ThumbnailManager::windowRemoved, this, &NdeTaskBar::onWindowRemoved);
    connect(ThumbnailManager::getIns(), &ThumbnailManager::windowChanged, this, &NdeTaskBar::onWindowChanged);
    connect(KWindowSystem::self(), SIGNAL(currentDesktopChanged(int)), this, SLOT(onDesktopChanged(int)));
    connect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)), this, SLOT(onActiveWindowChanged(WId)));
//	QStringList fixedApps = mPlugin->settings()->value("FixedApps").toStringList();
//    for (int i = 0; i < fixedApps.size() ; i++) {
//        XdgDesktopFile xdg;
//        if (xdg.load(fixedApps.at(i)))
//        {
//            if (xdg.isSuitable(false, "X-LXQT"))
//            {
//                addButton(xdg);
//            }
//        }
//    }
}

/************************************************

 ************************************************/
NdeTaskBar::~NdeTaskBar()
{
    delete mStyle;
}

/************************************************

 ************************************************/
bool NdeTaskBar::acceptWindow(WId window) const
{
    QFlags<NET::WindowTypeMask> ignoreList;
    ignoreList |= NET::DesktopMask;
    ignoreList |= NET::DockMask;
    ignoreList |= NET::SplashMask;
    ignoreList |= NET::ToolbarMask;
    ignoreList |= NET::MenuMask;
    ignoreList |= NET::PopupMenuMask;
    ignoreList |= NET::NotificationMask;

    KWindowInfo info(window, NET::WMWindowType | NET::WMState, NET::WM2TransientFor);
    if (!info.valid())
        return false;

    if (NET::typeMatchesMask(info.windowType(NET::AllTypesMask), ignoreList))
        return false;

    if (info.state() & NET::SkipTaskbar)
        return false;

    // WM_TRANSIENT_FOR hint not set - normal window
    WId transFor = info.transientFor();
    if (transFor == 0 || transFor == window || transFor == (WId) QX11Info::appRootWindow())
        return true;

    info = KWindowInfo(transFor, NET::WMWindowType);

    QFlags<NET::WindowTypeMask> normalFlag;
    normalFlag |= NET::NormalMask;
    normalFlag |= NET::DialogMask;
    normalFlag |= NET::UtilityMask;

    return !NET::typeMatchesMask(info.windowType(NET::AllTypesMask), normalFlag);
}

/************************************************

 ************************************************/
void NdeTaskBar::dragEnterEvent(QDragEnterEvent* event)
{
    if (event->mimeData()->hasFormat(NdeTaskGroup::mimeDataFormat()))
    {
        event->acceptProposedAction();
        buttonMove(nullptr, qobject_cast<NdeTaskGroup *>(event->source()), event->pos());
    } else
    if (event->mimeData()->hasUrls())
    {
        event->acceptProposedAction();
    }
    QWidget::dragEnterEvent(event);
}

/************************************************

 ************************************************/
void NdeTaskBar::dragMoveEvent(QDragMoveEvent * event)
{
    //we don't get any dragMoveEvents if dragEnter wasn't accepted
    buttonMove(nullptr, qobject_cast<NdeTaskGroup *>(event->source()), event->pos());
    QWidget::dragMoveEvent(event);
}
void NdeTaskBar::dropEvent(QDropEvent *event)
{
    const auto urls = event->mimeData()->urls().toSet();
    for (const QUrl &url : urls)
    {
        QString fileName(url.isLocalFile() ? url.toLocalFile() : url.url());

        XdgDesktopFile xdg;
        XdgMenu menu;
        menu.setEnvironments(QStringList() << QSL("X-LXQT") << QSL("LXQt"));
        if (xdg.load(fileName))
        {
            if (xdg.isSuitable(false, "X-LXQT"))
            {
                xdg.startDetached();
//                addButton(xdg);
            }
        }
    }
}
/************************************************

 ************************************************/
void NdeTaskBar::buttonMove(NdeTaskGroup * dst, NdeTaskGroup * src, QPoint const & pos)
{
    int src_index;
    if (!src || -1 == (src_index = mTasksLayout->indexOf(src)))
    {
        qDebug() << "Dropped invalid";
        return;
    }

    const int size = mTasksLayout->count();
    Q_ASSERT(0 < size);
    //dst is nullptr in case the drop occured on empty space in taskbar
    int dst_index;
    if (nullptr == dst)
    {
        //moving based on taskbar (not signaled by button)
        QRect occupied = mTasksLayout->occupiedGeometry();
        QRect last_empty_row{occupied};
        const QRect last_item_geometry = mTasksLayout->itemAt(size - 1)->geometry();
        if (mPlugin->panel()->isHorizontal())
        {
            if (isRightToLeft())
            {
                last_empty_row.setTopRight(last_item_geometry.topLeft());
            } else
            {
                last_empty_row.setTopLeft(last_item_geometry.topRight());
            }
        } else
        {
            if (isRightToLeft())
            {
                last_empty_row.setTopRight(last_item_geometry.topRight());
            } else
            {
                last_empty_row.setTopLeft(last_item_geometry.topLeft());
            }
        }
        if (occupied.contains(pos) && !last_empty_row.contains(pos))
            return;

        dst_index = size;
    } else
    {
        //moving based on signal from child button
        dst_index = mTasksLayout->indexOf(dst);
    }

    //moving lower index to higher one => consider as the QList::move => insert(to, takeAt(from))
    if (src_index < dst_index)
    {
        if (size == dst_index
                || src_index + 1 != dst_index)
        {
            --dst_index;
        } else
        {
            //switching positions of next standing
            const int tmp_index = src_index;
            src_index = dst_index;
            dst_index = tmp_index;
        }
    }

    if (dst_index == src_index
            || mTasksLayout->animatedMoveInProgress()
       )
        return;

    mTasksLayout->moveItem(src_index, dst_index, true);
    QMouseEvent mouseEvent(QEvent::MouseButtonRelease, pos, Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
    QApplication::sendEvent(src, &mouseEvent);
}

int NdeTaskBar::getPanelSize()
{
    int size = this->panel()->iconSize();
    if(static_cast<NdePanel*>( this->panel()))
    {
        size = static_cast<NdePanel*>( this->panel())->panelSize();
    }
    return size;
}

void NdeTaskBar::reCalButtonMinWidth()
{
    int taskBarLenght = mPlugin->panel()->isHorizontal() ? (this->width() - mPageWidget->width()):(this->height() - mPageWidget->height());
    if (mButtonStyle == Qt::ToolButtonIconOnly)
    {
        mButtonMinWidth = mButtonWidth;
    }
    else
    {
        //计算一下在mButtonWidth宽度时，能放下应用的数量。
        int maxCount = (taskBarLenght -mTasksLayout->margin()*2 + mTasksLayout->spacing()) / (mButtonWidth + mTasksLayout->spacing());
        //计算一下比maxCount多一时应该设置的应用按钮宽度，最后的-1确保能放下maxCount+1个应用
        mButtonMinWidth = (taskBarLenght -mTasksLayout->margin()*2 - maxCount * mTasksLayout->spacing())/(maxCount + 1)-1;
    }
}

void NdeTaskBar::reCalPage()
{
    int taskBarLenght = mPlugin->panel()->isHorizontal() ? (this->width() - mPageWidget->width()):(this->height() - mPageWidget->height());
    int countLenght = mTasksLayout->margin()*2;
    int taskLenght = mButtonMinWidth;
    int itemPage = 0;
    for(int i= 0; i < mTasksLayout->count(); i++)
    {
        QLayoutItem * item = mTasksLayout->itemAt(i);
        NdeTaskGroup* taskItem = qobject_cast<NdeTaskGroup*>(item->widget());
        if(taskItem)
        {
            //只处理当前窗口的应用
            if(taskItem->visibleButtonsCount() <= 0) continue;
            if(countLenght + taskLenght > taskBarLenght)
            {
                itemPage += 1;
                countLenght = mTasksLayout->margin()*2 + taskLenght;
            }
            else
            {
                countLenght = countLenght + taskLenght + mTasksLayout->spacing();
            }
            //更新每个应用按钮所在的页数
            taskItem->setCurrentPanelPage(itemPage);
            //根据当前激活的窗口来计算当前页面
            if(taskItem->checkedButton())
            {
                mCurrentPage = itemPage;
            }
        }
    }
    mCountPage = itemPage + 1;
    //若当前没有激活的窗口,则可能出现mCountPage == mCurrentPage的情况,注意避免
    if(mCurrentPage >= mCountPage)
    {
        mCurrentPage = mCountPage - 1;
    }
}

/************************************************

 ************************************************/
void NdeTaskBar::groupBecomeEmptySlot()
{
    NdeTaskGroup * const group = qobject_cast<NdeTaskGroup*>(sender());
    Q_ASSERT(group);

    for (auto i = mKnownWindows.begin(); mKnownWindows.end() != i; )
    {
        if (group == *i)
            i = mKnownWindows.erase(i);
        else
            ++i;
    }
    mTasksLayout->removeWidget(group);
    group->deleteLater();
}
QString  getCmdLine(WId pid)
{
    QString filePath = QString("/proc/%1/cmdline").arg(pid);
    QFile procFile(filePath);
    if(!procFile.open(QIODevice::ReadOnly))
    {
        qDebug() << "Open" << filePath << "Failed!";
        return "";
    }
    QByteArrayList cmdline = procFile.readAll().trimmed().split(' ');
    QByteArrayList cmd = cmdline.at(0).split('/');
    procFile.close();
    return cmd.at(cmd.size() -1).trimmed();
}
/************************************************

 ************************************************/
void NdeTaskBar::addWindow(WId window)
{
    const QString taskName = KWindowInfo(window, nullptr, NET::WM2WindowClass).windowClassClass();
    const QString group_id = mGroupingEnabled ? taskName : QString("%1").arg(window);
    NETWinInfo info(QX11Info::connection(), window, QX11Info::appRootWindow(), NET::Properties(NET::WM2AllProperties) & ~NET::WMIcon, NET::Properties2());

    NdeTaskGroup *group = nullptr;

//    if(mProcessPid.contains(info.pid()))
//    {
//        QString cmdline = mProcessPid.value(info.pid());

//        if(mFixedApp.contains(cmdline))
//        {
//            group = mFixedApp.value(cmdline);
//            group->setButtonStatus(NdeTaskButton::BUTTON_FIXED_AND_OPENED);
//            INdePanel::Position panelPosition = mPlugin->panel()->position();
//            group->setChecked(true);
//            group->updateButtonStyle(panelPosition);
//            group->setWindowId(window);
//        }
//    }
//    QStringList execStrs =  mFixedApp.keys();
//    for (int i = 0; i < execStrs.size(); i++) {
//        QString exec = execStrs.at(i);
//        if(exec.contains(getCmdLine(info.pid())))
//        {
//            group = mFixedApp.value(exec);
//            group->setButtonStatus(NdeTaskButton::BUTTON_FIXED_AND_OPENED);
//            INdePanel::Position panelPosition = mPlugin->panel()->position();
//            group->setChecked(true);
//            group->updateButtonStyle(panelPosition);
//            group->setWindowId(window);
//        }
//    }

    auto i_group = mKnownWindows.find(window);
    if (mKnownWindows.end() != i_group)
    {
        if ((*i_group)->groupName() == group_id)
            group = *i_group;
        else
            (*i_group)->onWindowRemoved(window);
    }

    //check if window belongs to some existing group
    if (!group && mGroupingEnabled)
    {
        for (auto i = mKnownWindows.cbegin(), i_e = mKnownWindows.cend(); i != i_e; ++i)
        {
            if ((*i)->groupName() == group_id)
            {
                group = *i;
                break;
            }
        }
    }

    if (!group)
    {
        group = new NdeTaskGroup(group_id, window, this);
        int iconSize = this->panel()->iconSize();
        group->setIconSize(QSize(iconSize,iconSize));
        connect(group, SIGNAL(groupBecomeEmpty(QString)), this, SLOT(groupBecomeEmptySlot()));
        connect(group, SIGNAL(visibilityChanged(bool)), this, SLOT(refreshPlaceholderVisibility()));
        connect(group, &NdeTaskButton::dragging, this, [this] (QObject * dragSource, QPoint const & pos) {
            buttonMove(qobject_cast<NdeTaskGroup *>(sender()), qobject_cast<NdeTaskGroup *>(dragSource), pos);
        });

        mTasksLayout->addWidget(group);
        group->setToolButtonsStyle(mButtonStyle);
    }
    mKnownWindows[window] = group;
    group->addWindow(window);

}
void NdeTaskBar::addProcessPid(const QString &fileName, const qint64 &pid)
{
    mProcessPid.insert(pid, fileName);
}

//void NdeTaskBar::addButton(const XdgDesktopFile &xdg)
//{
//    NdeTaskGroup *group = nullptr;
//    QString execStr = xdg.expandExecString().at(0);
//    if(mFixedApp.find(execStr) != mFixedApp.end())
//    {
//        return;
//    }
//    QList<WId> winIds =  mKnownWindows.keys();
//    for (int i = 0; i < winIds.size(); i++) {
//        NETWinInfo info(QX11Info::connection(), winIds.at(i), QX11Info::appRootWindow(), NET::Properties(NET::WM2AllProperties) & ~NET::WMIcon, NET::Properties2());
//        if(execStr.contains(getCmdLine(info.pid())))
//        {
//            return ;
//        }
//    }
//    group = new NdeTaskGroup(xdg, this);
//    connect(group, SIGNAL(visibilityChanged(bool)), this, SLOT(refreshPlaceholderVisibility()));
//    connect(group, &NdeTaskButton::dragging, this, [this] (QObject * dragSource, QPoint const & pos) {
//        buttonMove(qobject_cast<NdeTaskGroup *>(sender()), qobject_cast<NdeTaskGroup *>(dragSource), pos);
//    });

//    mTasksLayout->addWidget(group);
//    group->setToolButtonsStyle(mButtonStyle);
////    mFixedApp[execStr] = group;
////    if(!group->isFixed())
////    {
////        QStringList fixedApps = mPlugin->settings()->value("FixedApps").toStringList();
////        fixedApps.append(xdg.fileName());
////        mPlugin->settings()->setValue("FixedApps", fixedApps);
////    }
//}
//void NdeTaskBar::removeGroupButton(const QString execStr)
//{
//    NdeTaskGroup *group = mFixedApp.value(execStr);
//    mFixedApp.remove(execStr);
//    qDebug() << group->buttonStatus();
//    if(group->buttonStatus() == NdeTaskButton::BUTTON_FIXED)
//    {
//        mTasksLayout->removeWidget(group);
//        group->deleteLater();
//    }
//}

/************************************************

 ************************************************/
auto NdeTaskBar::removeWindow(windowMap_t::iterator pos) -> windowMap_t::iterator
{
    WId const window = pos.key();
    NdeTaskGroup * const group = *pos;
    auto ret = mKnownWindows.erase(pos);
//    if(group->isFixed() && group->buttonStatus() == NdeTaskButton::BUTTON_FIXED_AND_OPENED)
//    {
//        group->updateButtonStyle(-1);
//        group->getNextPrevChildButton(true, false);
//        group->setButtonStatus(NdeTaskButton::BUTTON_FIXED);
//    }
    group->onWindowRemoved(window);

    return ret;
}

/************************************************

 ************************************************/
void NdeTaskBar::refreshTaskList()
{
    QList<WId> new_list;
    // Just add new windows to groups, deleting is up to the groups
    const auto wnds = KWindowSystem::stackingOrder();
    for (auto const wnd: wnds)
    {
        if (acceptWindow(wnd))
        {
            new_list << wnd;
            addWindow(wnd);
        }
    }

    //emulate windowRemoved if known window not reported by KWindowSystem
    for (auto i = mKnownWindows.begin(), i_e = mKnownWindows.end(); i != i_e; )
    {
        if (0 > new_list.indexOf(i.key()))
        {
            i = removeWindow(i);
        } else
            ++i;
    }

    refreshPlaceholderVisibility();
}

/************************************************

 ************************************************/
void NdeTaskBar::onWindowChanged(WId window, NET::Properties prop, NET::Properties2 prop2)
{
    auto i = mKnownWindows.find(window);
    if (mKnownWindows.end() != i)
    {
        if (!(*i)->onWindowChanged(window, prop, prop2) && acceptWindow(window))
        { // window is removed from a group because of class change, so we should add it again
            addWindow(window);
        }
    }
}

void NdeTaskBar::onWindowAdded(WId window)
{
    auto const pos = mKnownWindows.find(window);
    if (mKnownWindows.end() == pos && acceptWindow(window))
        addWindow(window);

    //添加一个窗口后需要判断一下
    reCalPage();
    realign();
}

/************************************************

 ************************************************/
void NdeTaskBar::onWindowRemoved(WId window)
{
    auto const pos = mKnownWindows.find(window);
    if (mKnownWindows.end() != pos)
    {
        //这里必须这样做，比如打开多个nde-config时，平铺模式下会崩溃。
        //nde-config只允许一个实例，多次打开nde-config就会删除新打开的实例，
        //导致刚刚新建的group被释放，其中的单线程函数changeLayout()执行时出错，
        //故：这里删除时用新的线程处理确保在以上线程之后处理删除
        QTimer::singleShot(0, [this,pos](){
            removeWindow(pos);
            //删除一个窗口后需要判断一下
            reCalPage();
            realign();
        });

    }
}

void NdeTaskBar::onActiveWindowChanged(WId window)
{
    //计算页面
    reCalPage();
    realign();
}

void NdeTaskBar::onDesktopChanged(int number)
{
    //切换桌面时，重新计算页面数据
    mCountPage = 1;
    mCurrentPage = 0;

    reCalPage();
    realign();
}

/************************************************

 ************************************************/
void NdeTaskBar::refreshButtonRotation()
{
    bool autoRotate = mAutoRotate && (mButtonStyle != Qt::ToolButtonIconOnly);

    INdePanel::Position panelPosition = mPlugin->panel()->position();
    emit buttonRotationRefreshed(autoRotate, panelPosition);
}

/************************************************

 ************************************************/
void NdeTaskBar::refreshPlaceholderVisibility()
{
    realign();

//    // if no visible group button show placeholder widget
//    bool haveVisibleWindow = false;
//    for (auto i = mKnownWindows.cbegin(), i_e = mKnownWindows.cend(); i_e != i; ++i)
//    {
//        if ((*i)->isVisible())
//        {
//            haveVisibleWindow = true;
//            break;
//        }
//    }
////    if(mFixedApp.size() > 0)
////        haveVisibleWindow = true;
}

/************************************************

 ************************************************/
void NdeTaskBar::setButtonStyle(Qt::ToolButtonStyle buttonStyle)
{
    const Qt::ToolButtonStyle old_style = mButtonStyle;
    mButtonStyle = buttonStyle;
    if (old_style != mButtonStyle)
        emit buttonStyleRefreshed(mButtonStyle);
}

/************************************************

 ************************************************/
void NdeTaskBar::settingsChanged()
{
    bool groupingEnabledOld = mGroupingEnabled;
//    bool showOnlyOneDesktopTasksOld = mShowOnlyOneDesktopTasks;
    const int showDesktopNumOld = mShowDesktopNum;
//    bool showOnlyCurrentScreenTasksOld = mShowOnlyCurrentScreenTasks;
//    bool showOnlyMinimizedTasksOld = mShowOnlyMinimizedTasks;
    const bool iconByClassOld = mIconByClass;

//    mButtonWidth = mPlugin->settings()->value("buttonWidth", 48).toInt();
//    mButtonHeight = mPlugin->settings()->value("buttonHeight", 42).toInt();
    QString s = mPlugin->settings()->value("buttonStyle").toString().toUpper();

    if (s == "ICON")
    {
        mButtonWidth = mPlugin->settings()->value("buttonWidth", 48).toInt();
        mButtonHeight = mPlugin->settings()->value("buttonHeight", 42).toInt();
        setButtonStyle(Qt::ToolButtonIconOnly);
    }
    else if (s == "TEXT"){
        mButtonWidth = 160;//mPlugin->settings()->value("buttonWidth", 250).toInt();
        mButtonHeight = mPlugin->settings()->value("buttonHeight", 42).toInt();
        setButtonStyle(Qt::ToolButtonTextOnly);
    }
    else{
        mButtonWidth = 160;//mPlugin->settings()->value("buttonWidth", 250).toInt();
        mButtonHeight = mPlugin->settings()->value("buttonHeight", 42).toInt();
        setButtonStyle(Qt::ToolButtonTextBesideIcon);
    }

//    mShowOnlyOneDesktopTasks = mPlugin->settings()->value("showOnlyOneDesktopTasks", mShowOnlyOneDesktopTasks).toBool();
    mShowDesktopNum = mPlugin->settings()->value("showDesktopNum", mShowDesktopNum).toInt();
//    mShowOnlyCurrentScreenTasks = mPlugin->settings()->value("showOnlyCurrentScreenTasks", mShowOnlyCurrentScreenTasks).toBool();
//    mShowOnlyMinimizedTasks = mPlugin->settings()->value("showOnlyMinimizedTasks", mShowOnlyMinimizedTasks).toBool();
    mAutoRotate = mPlugin->settings()->value("autoRotate", true).toBool();
    mCloseOnMiddleClick = mPlugin->settings()->value("closeOnMiddleClick", true).toBool();
    mRaiseOnCurrentDesktop = mPlugin->settings()->value("raiseOnCurrentDesktop", false).toBool();
//    mGroupingEnabled = mPlugin->settings()->value("groupingEnabled",true).toBool();
    //平铺模式下，每个应用单独显示
    mGroupingEnabled = mButtonStyle==Qt::ToolButtonIconOnly ? true:false;
    mShowGroupOnHover = mPlugin->settings()->value("showGroupOnHover",true).toBool();
    mIconByClass = mPlugin->settings()->value("iconByClass", false).toBool();
    mCycleOnWheelScroll = mPlugin->settings()->value("cycleOnWheelScroll", true).toBool();

    // Delete all groups if grouping feature toggled and start over
    if (groupingEnabledOld != mGroupingEnabled)
    {
        for (int i = mTasksLayout->count() - 1; 0 <= i; --i)
        {
            NdeTaskGroup * group = qobject_cast<NdeTaskGroup*>(mTasksLayout->itemAt(i)->widget());
            if (nullptr != group)
            {
                mTasksLayout->takeAt(i);
                group->deleteLater();
            }
        }
        mKnownWindows.clear();
    }

//    if (showOnlyOneDesktopTasksOld != mShowOnlyOneDesktopTasks
//            || (mShowOnlyOneDesktopTasks && showDesktopNumOld != mShowDesktopNum)
//            || showOnlyCurrentScreenTasksOld != mShowOnlyCurrentScreenTasks
//            || showOnlyMinimizedTasksOld != mShowOnlyMinimizedTasks
//            )
    if (mShowOnlyOneDesktopTasks && showDesktopNumOld != mShowDesktopNum)
        emit showOnlySettingChanged();
    if (iconByClassOld != mIconByClass)
        emit iconByClassChanged();

    reCalButtonMinWidth();
    //需要重新计算一下页面
    reCalPage();

    refreshTaskList();
}

/************************************************

 ************************************************/
void NdeTaskBar::realign()
{
    for (auto i = mKnownWindows.begin(); mKnownWindows.end() != i; i++)
    {
        i.value()->realign();
    }

    mTasksLayout->setEnabled(false);

    refreshButtonRotation();

    INdePanel *panel = mPlugin->panel();
    QSize maxSize = QSize(mButtonWidth, mButtonHeight);
    QSize minSize = QSize(mButtonMinWidth, mButtonHeight);
    int size = getPanelSize();

    bool rotated = false;
    if (panel->isHorizontal())
    {
        //翻页按钮布局
        mPageWidget->setFixedSize(size/2, size);
        mPageLayout->setDirection(QBoxLayout::TopToBottom);
        mPrviBtn->setOrigin(Qt::TopLeftCorner);
        mNextBtn->setOrigin(Qt::TopLeftCorner);

        //任务布局
        mTasksLayout->setRowCount(1);
        mTasksLayout->setColumnCount(0);
        mTasksLayout->setCellMinimumSize(minSize);
        mTasksLayout->setCellMaximumSize(maxSize);
        mTasksLayout->setDirection(rotated ? Nde::GridLayout::TopToBottom : Nde::GridLayout::LeftToRight);

        //整体布局
        mLayout->setDirection(QBoxLayout::LeftToRight);
    }
    else
    {
        //翻页按钮布局
        mPageWidget->setFixedSize(size, size/2);
        mPageLayout->setDirection(QBoxLayout::LeftToRight);
        mPrviBtn->setOrigin(Qt::BottomLeftCorner);
        mNextBtn->setOrigin(Qt::BottomLeftCorner);

        //任务布局
        mTasksLayout->setRowCount(0);
        if (mButtonStyle == Qt::ToolButtonIconOnly)
        {
            // Vertical + Icons
            mTasksLayout->setColumnCount(1);
        }
        else
        {
            rotated = mAutoRotate && (panel->position() == INdePanel::PositionLeft || panel->position() == INdePanel::PositionRight);

            // Vertical + Text
            if (rotated)
            {
                maxSize.rwidth()  = mButtonHeight;
                maxSize.rheight() = mButtonWidth;
                minSize.rwidth()  = mButtonHeight;
                minSize.rheight() = mButtonMinWidth;

                mTasksLayout->setColumnCount(1);
            }
            else
            {
                mTasksLayout->setColumnCount(1);
            }
        }
        mTasksLayout->setCellMinimumSize(minSize);
        mTasksLayout->setCellMaximumSize(maxSize);
        mTasksLayout->setDirection(rotated ? Nde::GridLayout::TopToBottom : Nde::GridLayout::LeftToRight);

        //整体布局
        mLayout->setDirection(QBoxLayout::TopToBottom);
    }

    //panel方向改变时需要重新计算
    if(mCurrentPosition != panel->position())
    {
        reCalButtonMinWidth();
        reCalPage();
        mCurrentPosition = panel->position();
    }

    for(int i= 0; i < mTasksLayout->count(); i++)
    {
        QLayoutItem * item = mTasksLayout->itemAt(i);
        NdeTaskGroup* taskItem = qobject_cast<NdeTaskGroup*>(item->widget());
        if(taskItem)
        {
            //只处理当前窗口的应用
            if(taskItem->visibleButtonsCount() <= 0) continue;
            //激活当前应用按下的状态，有时这个状态会丢失
            if(taskItem->checkedButton()) taskItem->setChecked(true);
            if(taskItem->getCurrentPanelPage() == mCurrentPage)
            {
                taskItem->show();
            }
            else {
                taskItem->hide();
            }
        }
    }

    //根据页面数量，设置翻译按钮显示状态
    mPageWidget->setVisible(mCountPage > 1);

    mTasksLayout->setEnabled(true);

    //our placement on screen could have been changed
//    emit showOnlySettingChanged();
    emit refreshIconGeometry();
}

/************************************************

 ************************************************/
void NdeTaskBar::wheelEvent(QWheelEvent* event)
{
    if (!mCycleOnWheelScroll)
        return QFrame::wheelEvent(event);

    static int threshold = 0;
    threshold += abs(event->delta());
    if (threshold < 300)
        return QFrame::wheelEvent(event);
    else
        threshold = 0;

    int delta = event->delta() < 0 ? 1 : -1;

    // create temporary list of visible groups in the same order like on the layout
    QList<NdeTaskGroup*> list;
    NdeTaskGroup *group = NULL;
    for (int i = 0; i < mTasksLayout->count(); i++)
    {
        QWidget * o = mTasksLayout->itemAt(i)->widget();
        NdeTaskGroup * g = qobject_cast<NdeTaskGroup *>(o);
        if (!g)
            continue;

        if (g->isVisible())
            list.append(g);
        if (g->isChecked())
            group = g;
    }

    if (list.isEmpty())
        return QFrame::wheelEvent(event);

    if (!group)
        group = list.at(0);

    NdeThumbnailButton *button = NULL;

    while (!button)
    {
        button = group->getNextPrevChildButton(delta == 1, !(list.count() - 1));
        if (button)
            button->raiseApplication();
        int idx = (list.indexOf(group) + delta + list.count()) % list.count();
        group = list.at(idx);
    }
    QFrame::wheelEvent(event);
}

/************************************************

 ************************************************/
void NdeTaskBar::resizeEvent(QResizeEvent* event)
{
    reCalButtonMinWidth();
    reCalPage();
    realign();

    emit refreshIconGeometry();
    return QWidget::resizeEvent(event);
}

/************************************************

 ************************************************/
void NdeTaskBar::changeEvent(QEvent* event)
{
    // if current style is changed, reset the base style of the proxy style
    // so we can apply the new style correctly to task buttons.
    if(event->type() == QEvent::StyleChange)
        mStyle->setBaseStyle(NULL);

    QFrame::changeEvent(event);
}

/************************************************

 ************************************************/
void NdeTaskBar::registerShortcuts()
{
    // Register shortcuts to switch to the task
    // mPlaceHolder is always at position 0
    // tasks are at positions 1..10
    GlobalKeyShortcut::Action * gshortcut;
    QString path;
    QString description;
    for (int i = 1; i <= 10; ++i)
    {
        path = QString("/panel/%1/task_%2").arg(mPlugin->settings()->group()).arg(i);
        description = tr("Activate task %1").arg(i);

        gshortcut = GlobalKeyShortcut::Client::instance()->addAction(QStringLiteral(), path, description, this);

        if (nullptr != gshortcut)
        {
            mKeys << gshortcut;
            connect(gshortcut, &GlobalKeyShortcut::Action::registrationFinished, this, &NdeTaskBar::shortcutRegistered);
            connect(gshortcut, &GlobalKeyShortcut::Action::activated, mSignalMapper, static_cast<void (QSignalMapper::*)()>(&QSignalMapper::map));
            mSignalMapper->setMapping(gshortcut, i);
        }
    }
}

void NdeTaskBar::shortcutRegistered()
{
    GlobalKeyShortcut::Action * const shortcut = qobject_cast<GlobalKeyShortcut::Action*>(sender());

    disconnect(shortcut, &GlobalKeyShortcut::Action::registrationFinished, this, &NdeTaskBar::shortcutRegistered);

    const int i = mKeys.indexOf(shortcut);
    Q_ASSERT(-1 != i);

    if (shortcut->shortcut().isEmpty())
    {
        // Shortcuts come in order they were registered
        // starting from index 0
        const int key = (i + 1) % 10;
        shortcut->changeShortcut(QStringLiteral("Meta+%1").arg(key));
    }
}

void NdeTaskBar::activateTask(int pos)
{
    for (int i = 1; i < mTasksLayout->count(); ++i)
    {
        QWidget * o = mTasksLayout->itemAt(i)->widget();
        NdeTaskGroup * g = qobject_cast<NdeTaskGroup *>(o);
        if (g && g->isVisible())
        {
            pos--;
            if (pos == 0)
            {
                if(g->isChecked())
                {
                    g->minimizeApplication();
                }
                else
                {
                    g->raiseApplication();
                }
                break;
            }
        }
    }
}

void NdeTaskBar::slotPrviBtnClicked()
{
    //
    if(mCurrentPage > 0) {
        mCurrentPage -= 1;
        realign();
    }
}

void NdeTaskBar::slotNextBtnClicked()
{
    //
    if(mCurrentPage < mCountPage - 1 ) {
        mCurrentPage += 1;
        realign();
    }
}
