#ifndef NDETHUMBNAILBUTTON_H
#define NDETHUMBNAILBUTTON_H

#include <QTimer>
#include "../panel/indepanel.h"
#include <QFrame>
#include <QToolButton>

class QLabel;
class NdeTaskBar;
class CloseButton: public QToolButton{
    Q_OBJECT
public:
    CloseButton(QWidget *parent = nullptr);
signals:
    void closeApp();
protected:
    void mouseReleaseEvent(QMouseEvent *event) override;
    void enterEvent(QEvent *) override;
    void leaveEvent(QEvent *) override;
    void mousePressEvent(QMouseEvent *) override;
};
class NdeThumbnailButton : public QFrame
{
    Q_OBJECT
public:
    explicit NdeThumbnailButton(const WId window, NdeTaskBar * taskBar, QWidget *parent = nullptr);
    virtual ~NdeThumbnailButton() override;

    void updateTitle();
    QString title() const;
    bool isApplicationActive() const;
    bool isOnDesktop(int desktop) const;
    bool hasUrgencyHint() const { return mUrgencyHint; }
    void setUrgencyHint(bool set);
    WId windowId() const { return mWindow; }
    bool isOnCurrentScreen() const;
    NdeTaskBar * parentTaskBar() const {return mParentTaskBar;}
    void addThumbnail(QPixmap pixmap);
    virtual void setAutoRotation(bool value, INdePanel::Position position);
signals:
    void clicked();
public slots:
    void closeApplication();
    void raiseApplication();
    void minimizeApplication();

    void setOrigin(Qt::Corner);
protected:
    void paintEvent(QPaintEvent *event) override;
    void enterEvent(QEvent * event) override;
    virtual void leaveEvent(QEvent * event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
private:
    WId mWindow;
    NdeTaskBar * mParentTaskBar;
    CloseButton *closeButton;
    QLabel *windowTitle;
    QLabel *thumbnailPixmap;
    bool mUrgencyHint;
    Qt::Corner mOrigin;
    enum ButtonStatus{
        BUTTON_NORMAL,
        BUTTON_HOVER
    };
    static const int TITLE_HEIGHT = 22;
    ButtonStatus mButtonStatus;
};

typedef QHash<WId,NdeThumbnailButton*> NdeThumbnailButtonHash;
#endif
