#include "calendarwidget.h"
#include <QPainter>
#include <KWindowEffects>
#include <QDebug>
#include <QTimer>
#include <QFile>
#include <QJsonDocument>
#include <QJsonArray>
#include <QJsonObject>
#include <Nde/RotatedWidget>
#include <Nde/Settings>
#include <QStyleOption>
#include <QBitmap>
#include <QDBusConnection>
#include <QDBusInterface>
#include "../panel/ndepanel.h"

#define T_D_YAER   1924 //  天干地支开始计算年月
bool ClickToday = false;
DirectWidget * selectdirect = nullptr;
DateLabel * slectlabel = nullptr;
QDate globaldate = QDate::currentDate();
//每年春节对应的公历日期
static const int springFestival[] = {
    130,217,206,                                // 1968,1969,1970
    127,215,203,123,211,131,218,207,128,216,    // 1971--1980
    205,125,213,202,220,209,219,217,206,127,    // 1981---1990
    215,204,123,210,131,219,207,128,216,205,    // 1991--2000
    124,212,201,122,209,129,218,207,126,214,    // 2001--2010
    203,123,210,131,219,208,128,216,205,125,    // 2011--2020                                       //2011--2020
    212,201,122,210,129,217,206,126,213,203,    // 2021--2030
    123,211,131,219,208,128,215,204,124,212     // 2031--2040
};

//16--18位表示闰几月，0--12位表示农历每月的数据，高位表示1月，低位表示12月（农历闰月就会多一个月）
static const int  nLunarData[] = {
    461653,1386,2413,                                            // 1968,1969,1970
    330077,1197,2637,268877,3365,531109,2900,2922,398042,2395,   // 1971--1980
    1179,267415,2635,661067,1701,1748,398772,2742,2391,330031,   // 1981---1990
    1175,1611,200010,3749,527717,1452,2742,332397,2350,3222,     // 1991--2000
    268949,3402,3493,133973,1386,464219,605,2349,334123,2709,    // 2001--2010
    2890,267946,2773,592565,1210,2651,395863,1323,2707,265877,   // 2011--2020
    1706,2773,133557,1206,398510,2638,3366,335142,3411,1450,     // 2021 -- 2030
    200042,2413,723293,1197,2637,399947,3365,3410,334676,2906    // 2031 -- 2040
};

static const char *s_pchDayName[] =  {
    ("*"),
    ("初一"), ("初二"), ("初三"), ("初四"), ("初五"),
    ("初六"), ("初七"), ("初八"), ("初九"), ("初十"),
    ("十一"), ("十二"), ("十三"), ("十四"), ("十五"),
    ("十六"), ("十七"), ("十八"), ("十九"), ("二十"),
    ("廿一"), ("廿二"), ("廿三"), ("廿四"), ("廿五"),
    ("廿六"), ("廿七"), ("廿八"), ("廿九"), ("三十")
};

/*农历月份名*/
static const char *s_pchMonName[] = {
    ("*"),
    ("正月"), ("二月"), ("三月"), ("四月"),
    ("五月"), ("六月"), ("七月"), ("八月"),
    ("九月"), ("十月"), ("冬月"), ("腊月")
};

/*公历每月前面的天数*/
int monthAdd[] = {
    0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
};



Date::Date(QObject *parent) : QObject(parent)
{

}

/**
 * @brief Date::getLunarDate 计算农历
 * @param year  年
 * @param month 月
 * @param day 日
 * @return
 */
QString Date::getLunarDate (int year, int month, int day){
    int nTheDate,nIsEnd,nMonTemp,k,n,nBit;
    // 先获取公历节日
    QString strDate;

    /*现在计算农历：获得当年春节的公历日期（比如：2015年春节日期为（2月19日）），
            以此为分界点，2.19前面的农历是2014年农历（用2014年农历数据来计算），
            2.19以及之后的日期，农历为2015年农历（用2015年农历数据来计算）。*/
    nMonTemp = year - 1968;
    if ((year <= 1968) || (year >=2040)) {
        return "";
    }
    int springFestivalMonth = springFestival[nMonTemp] / 100;
    int springFestivalDaty = springFestival[nMonTemp] % 100;

    if(month < springFestivalMonth )
    {
        nMonTemp--;
        nTheDate = 365 * 1 + day + monthAdd[month - 1] - 31 * ((springFestival[nMonTemp] / 100) - 1) - springFestival[nMonTemp] % 100 + 1;

        if((!(year % 4)) && (month > 2))
            nTheDate = nTheDate + 1;

        if((!((year - 1) % 4)))
            nTheDate = nTheDate + 1;
    }
    else if (month == springFestivalMonth)
    {
        if (day < springFestivalDaty) {
            nMonTemp--;
            nTheDate = 365 * 1 + day + monthAdd[month - 1] - 31 * ((springFestival[nMonTemp] / 100) - 1) - springFestival[nMonTemp] % 100 + 1;

            if((!(year % 4)) && (month > 2))
                nTheDate = nTheDate + 1;

            if((!((year-1) % 4)))
                nTheDate = nTheDate + 1;
        }
        else {
            nTheDate = day + monthAdd[month - 1] - 31 * ((springFestival[nMonTemp] / 100) - 1) - springFestival[nMonTemp] % 100 + 1;

            if((!(year % 4)) && (month > 2))
                nTheDate = nTheDate + 1;
        }
    }else{
        nTheDate = day + monthAdd[month - 1] - 31 * ((springFestival[nMonTemp] / 100) - 1) - springFestival[nMonTemp] % 100 + 1;
        if((!(year % 4)) && (month > 2))
            nTheDate = nTheDate + 1;
    }
    nIsEnd = 0;

    while(nIsEnd != 1)  {
        if(nLunarData[nMonTemp] < 4095)
            k = 11;
        else
            k = 12;
        n = k;
        while(n >= 0)   {
            nBit = nLunarData[nMonTemp];

            nBit = nBit >> n;
            nBit = nBit % 2;
            if (nTheDate <= (29 + nBit))    {
                nIsEnd = 1;
                break;
            }

            nTheDate = nTheDate - 29 - nBit;
            n = n - 1;
        }

        if(nIsEnd)
            break;

        nMonTemp = nMonTemp + 1;
    }

    // 农历的年月日
    year = 1969 + nMonTemp -1;
    month = k - n + 1;

    day = nTheDate;
    if (k == 12) {
        if (month >= (nLunarData[nMonTemp] / 65536) + 1)
            month = month - 1;
    }
    if (day == 1) {
        strDate = tr(s_pchMonName[month]);
    } else {
        strDate = tr(s_pchDayName[day]);
    }


    return strDate;
}

DateConfig& DateConfig::instance()
{
    static DateConfig* _self = nullptr;
    if (!_self) {
        _self = new DateConfig();
    }
    return *_self;
}

int DateConfig::getDate(QDate date)
{
   QString key = date.toString("yyyy-MM-dd");
   auto iter = worklist.find(key);
   if (iter != worklist.end()){
        return iter.value();
   } else {
       return -1;
   }
}

void DateConfig::Init()
{
    worklist.clear();
    QFile loadFile("/home/test/date.json");
    if (loadFile.exists()){
        loadFile.open(QIODevice::ReadOnly | QIODevice::Text);

        QString strCfg = QString::fromLocal8Bit(loadFile.readAll());
        QJsonDocument loadDoc = QJsonDocument::fromJson(strCfg.toUtf8());

        QJsonArray array;
        if (loadDoc.isArray()){
            array = loadDoc.array();
            int size = array.size();
            for (int i =0; i< size; i++){
                QJsonObject ob = array.at(i).toObject();
                QString date = ob.value("date").toString();
                int flag = ob.value("flag").toInt();
                worklist.insert(date,flag);
            }
        }
    }
}

void YearLabel::paintEvent(QPaintEvent *event){
        QPainter painter(this);
        if (year == QDate::currentDate().year()){
            QRect rect = this->rect();
            painter.setPen(Qt::NoPen);
            painter.setBrush(QColor("#2c9dfd"));
            //painter.drawRect(rect);
            painter.drawRoundRect(rect,25,25);
        } else if(moveon){
            QRect rect = this->rect();
            painter.setPen(Qt::NoPen);
            QColor color= QColor("#ffffff");
            color.setAlphaF(0.06);
            painter.setBrush(color);
            //painter.drawRect(rect);
            painter.drawRoundRect(rect,25,25);
       }
}


void monthLabel::paintEvent(QPaintEvent *event){
        QPainter painter(this);
        if ((month.year() == QDate::currentDate().year()) && (month.month() == QDate::currentDate().month())){
            QRect rect = this->rect();
            painter.setPen(Qt::NoPen);
            painter.setBrush(QColor("#2c9dfd"));
            //painter.drawRect(rect);
            painter.drawRoundRect(rect,25,25);
        } else if(moveon){
            QRect rect = this->rect();
            painter.setPen(Qt::NoPen);
            QColor color= QColor("#ffffff");
            color.setAlphaF(0.06);
            painter.setBrush(color);
            //painter.drawRect(rect);
            painter.drawRoundRect(rect,25,25);
       }
}



DateLabel::DateLabel(QWidget * parent)
    :QWidget(parent)
{
    setFixedSize(57,57);
    new_date = new QLabel();
    old_date = new QLabel();
    new_date->setFixedHeight(18);
    old_date->setFixedHeight(15);
    new_date->setAlignment(Qt::AlignCenter);
    old_date->setAlignment(Qt::AlignCenter);
}

void DateLabel::setDate(QDate date,QDate curdate,bool d)
{

    QDate today = QDate::currentDate();
    if (today == date){
        bcurday = true;
    } else {
        bcurday = false;
    }

    m_date = date;
    if (date == curdate) {
        bselected = true;
        slectlabel = this;
    } else {
        bselected = false;
    }
    int flag = DateConfig::instance().getDate(date);
    if (flag ==1){
        bwork = true;
        bholiday = false;
    } else if (flag == 0) {
        bwork = false;
        bholiday = true;
    } else {
        bwork = false;
        bholiday = false;
    }
    dark = d;
    if (dark) {
        new_date->setStyleSheet("dateWidget QLabel{font-size:14px;color:rgba(255,255,255,50)}");
        old_date->setStyleSheet("dateWidget QLabel{font-size:14px;color:rgba(255,255,255,50)}");
    } else {
        new_date->setStyleSheet("dateWidget QLabel{font-size:18px;color:rgb(255,255,255)}");
        old_date->setStyleSheet("dateWidget QLabel{font-size:14px;color:rgba(255,255,255,100)}");
    }
    if (bcurday) {
        old_date->setStyleSheet("dateWidget QLabel{font-size:14px;color:rgb(255,255,255)}");
    } else {

    }
    //
    new_date->setText(QString::number(date.day()));
    //
    old_date->setText(Date::getLunarDate(date.year(),date.month(),date.day()));
    repaint();
}

void DateLabel::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    if(verLayoutdate) {
        delete verLayoutdate;
    }

    if (bcurday) {
        QRect rect = this->rect();
        painter.setPen(Qt::NoPen);
        painter.setBrush(QColor("#2c9dfd"));
        //painter.drawRect(rect);
        painter.drawRoundRect(rect,25,25);

    } else if (bselected) {
        QRect rect = this->rect();
        painter.setPen(Qt::NoPen);
        QColor color = QColor("#ffffff");
        color.setAlphaF(0.1);
        painter.setBrush(color);
        //painter.drawRect(rect);
        painter.drawRoundRect(rect,25,25);
    } else if (moveon) {
        QRect rect = this->rect();
        painter.setPen(Qt::NoPen);
        QColor color = QColor("#ffffff");
        color.setAlphaF(0.06);
        painter.setBrush(color);
        //painter.drawRect(rect);
        painter.drawRoundRect(rect,25,25);
    }
    verLayoutdate = new QVBoxLayout(this);
    verLayoutdate->setAlignment(Qt::AlignCenter);
    if(!bcurday)
        verLayoutdate->addStretch();
    verLayoutdate->addWidget(new_date);
    verLayoutdate->addWidget(old_date);


    QRect day = QRect(0,0,14,16);
    QRect txt = QRect(2,0,14,16);
    if(bwork) {
        painter.setPen(Qt::NoPen);
        painter.setBrush(QColor("#ff7800"));
        //painter.drawRect(rect);
        painter.drawRoundRect(day,25,25);

        QFont ft1;
        ft1.setPointSize(7);
        painter.setFont(ft1);
        painter.setPen(Qt::white);
        painter.setBrush(Qt::NoBrush);
        QTextOption option(Qt::AlignLeft);
        option.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
        painter.drawText(txt, "班",option);
    }
    if (bholiday) {
        painter.setPen(Qt::NoPen);
        painter.setBrush(QColor("#f75262"));
        //painter.drawRect(rect);
        painter.drawRoundRect(day,25,25);

        QFont ft1;
        ft1.setPointSize(7);
        painter.setFont(ft1);
        painter.setPen(Qt::white);
        painter.setBrush(Qt::NoBrush);
        QTextOption option(Qt::AlignLeft);
        option.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
        painter.drawText(txt, "休",option);
    }

}

void YMLabel::setSelected(bool selected)
{
   if (selected) {
       setStyleSheet("background: transparent;border:none;font-size:18px;color:#0082f0");
   } else {
       setStyleSheet("background: transparent;border:none;font-size:18px;color:#ffffff");
   }

}
void YMLabel::mousePressEvent(QMouseEvent * event)
{
    emit clicked();
}

void DateLabel::mousePressEvent(QMouseEvent *event)
{
    if (slectlabel) {
        slectlabel->setSelect(false);
    }
    slectlabel = this;
    setSelect(true);
    emit selectdate(m_date);
    //repaint();
}


void DirectWidget::setSelected(bool f)
{
    repaint();
}
void DirectWidget::mousePressEvent(QMouseEvent * event)
{
    selected = true;
    emit clicked();
    repaint();
}
void DirectWidget::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    QRect rect = this->rect();
    if (selected ){
        QPen pen(QColor("#0082f0"),1);
        painter.setPen(pen);
    } else if (hover) {
        QPen pen(QColor("#2c9dfc"),1);
        painter.setPen(pen);
    } else {
        QPen pen(QColor("#ffffff"),1);
        painter.setPen(pen);
    }

    painter.setBrush(Qt::NoBrush);
    if (direction == 0){
        painter.drawLine(0,0,rect.width(),rect.height()/2);
        painter.drawLine(rect.width(),rect.height()/2,0,rect.height());
    } else {
        painter.drawLine(rect.width(),0,0,rect.height()/2);
        painter.drawLine(0,rect.height()/2,rect.width(),rect.height());
    }

}

DateTitle::DateTitle(QWidget * parent):
    QWidget(parent)
{
    setFixedHeight(68);
    Init();
}

void DateTitle::paintEvent(QPaintEvent *event)
{
    if(verLayouttime){
        delete verLayouttime;
    }
    verLayouttime = new QVBoxLayout(this);
    verLayouttime->setAlignment(Qt::AlignCenter);
    verLayouttime->addWidget(lab_time);
    verLayouttime->addStretch();
    verLayouttime->addWidget(lab_date);
}

void DateTitle::Init()
{
    verLayouttime = new QVBoxLayout(this);
    //verLayouttime->setAlignment(Qt::AlignCenter);
    lab_time = new QLabel();
    lab_time->setFixedHeight(30);
    lab_time->setStyleSheet("dateWidget QLabel{font-size:30px;color:rgb(255,255,255)}");
    lab_time->setText("10:50:45");

    lab_date = new QLabel();
    lab_date->setFixedHeight(14);
    lab_date->setStyleSheet("dateWidget QLabel{font-size:14px;color:rgba(255,255,255,100)}");
    lab_date->setText("2020/05/08");

    SetDateTime();
    InitTimer();

}

void DateTitle::DateTimer()
{
    SetDateTime();
    update();
}

void DateTitle::InitTimer()
{
    if (nullptr == timer){
        timer = new QTimer;
        timer->setSingleShot(false);
        timer->start(1000);
        connect(timer,SIGNAL(timeout()),this,SLOT(DateTimer()));
    }
}

void DateTitle::SetDateTime()
{
    QDateTime datetime = QDateTime::currentDateTime();
    QString time = datetime.time().toString();
    lab_time->setText(time);
    QString date = datetime.date().toString("yyyy/MM/dd");
    int day = datetime.date().dayOfWeek();
    lab_date->setText(date + "    " + s_strWeekl[day-1]);
}

DateToolbar::DateToolbar(QDate d,bool b,QWidget * parent):
    QWidget(parent),
    m_date(d),
    b_showym(b)
{
    setFixedHeight(30);
    Init();
}

void DateToolbar::paintEvent(QPaintEvent *event)
{
    paintitem();
}

void DateToolbar::paintitem()
{
    if(toolbarlay){
        delete toolbarlay;
    }
    toolbarlay = new QHBoxLayout();
    toolbarlay->addSpacing(30);
    if (b_showym){
        toolbarlay->addWidget(showym);
        toolbarlay->setAlignment(showym,Qt::AlignLeft);

    } else {
        toolbarlay->addWidget(pre_year);
        toolbarlay->setAlignment(pre_year,Qt::AlignLeft);

        toolbarlay->addSpacing(12);
        cur_year->setText(QString::number(m_date.year()));
        toolbarlay->addWidget(cur_year);
        toolbarlay->setAlignment(cur_year,Qt::AlignLeft);

        toolbarlay->addSpacing(12);
        toolbarlay->addWidget(next_year);
        toolbarlay->setAlignment(next_year,Qt::AlignLeft);

        toolbarlay->addSpacing(22);
        toolbarlay->addWidget(pre_month);
        toolbarlay->setAlignment(pre_month,Qt::AlignLeft);

        toolbarlay->addSpacing(12);
        if(m_date.month() < 10) {
            cur_month->setText("0" + QString::number(m_date.month()));
        } else {
            cur_month->setText(QString::number(m_date.month()));
        }
        toolbarlay->addWidget(cur_month);
        toolbarlay->setAlignment(cur_month,Qt::AlignLeft);

        toolbarlay->addSpacing(12);
        toolbarlay->addWidget(next_month);
        toolbarlay->setAlignment(next_month,Qt::AlignLeft);

    }
    toolbarlay->addStretch();
    toolbarlay->addWidget(today);
    toolbarlay->setAlignment(today,Qt::AlignRight);
    toolbarlay->addSpacing(30);
    this->setLayout(toolbarlay);
}

void DateToolbar::Init()
{
    toolbarlay = new QHBoxLayout(this);
    //m_date = QDate::currentDate();

    if(b_showym){
        showym = new QLabel(this);
        showym->setStyleSheet("background: transparent;border:none;font-size:18px;color:#ffffff");
    } else {
        pre_year = new DirectWidget(1,this);
        //pre_year->setText("<");
        connect(pre_year,SIGNAL(clicked()),this,SLOT(preyear()));

        cur_year = new YMLabel(this);
        cur_year->setFixedWidth(40);
        cur_year->setFixedHeight(18);
        cur_year->setText(QString::number(m_date.year()));
        connect(cur_year,SIGNAL(clicked()),this,SLOT(curyear()));

        next_year = new DirectWidget(0,this);
        connect(next_year,SIGNAL(clicked()),this,SLOT(nextyear()));

        pre_month = new DirectWidget(1,this);

        connect(pre_month,SIGNAL(clicked()),this,SLOT(premonth()));

        cur_month = new YMLabel(this);
        cur_month->setFixedHeight(18);
        if(m_date.month() < 10) {
            cur_month->setText("0" + QString::number(m_date.month()));
        } else {
            cur_month->setText(QString::number(m_date.month()));
        }

        connect(cur_month,SIGNAL(clicked()),this,SLOT(curmonth()));

        next_month = new DirectWidget(0,this);
        connect(next_month,SIGNAL(clicked()),this,SLOT(nextmonth()));

    }


    today = new todyLabel(this);
    today->setFixedHeight(18);
    today->setText("返回今天");

    connect(today,SIGNAL(clicked()),this,SLOT(ontoday()));



    paintitem();

}


CalendarWidget::CalendarWidget(QWidget *parent) : QWidget(parent)
{
    setFixedSize(438,440);
    Init();
}


void CalendarWidget::Init()
{
    verLayoutCalendar = new QVBoxLayout();


    widgetWeek = new QWidget(this);
    horLayoutWeek = new QHBoxLayout(widgetWeek);
    horLayoutWeek->setAlignment(Qt::AlignCenter);
    horLayoutWeek->setContentsMargins(30, 0, 30, 0);

    for (int i = 0; i < Week; i++) {
        labweek[i] = new QLabel(widgetWeek);
        labweek[i]->setText(s_strWeek[i]);
        labweek[i]->setStyleSheet("dateWidget QLabel{font-size:14px;color:rgb(255,255,255)}");
        labweek[i]->setMinimumHeight(14);
        //labweek[i]->setMinimumWidth(60);
        labweek[i]->setAlignment(Qt::AlignCenter);

        horLayoutWeek->addWidget(labweek[i]);
        if (i < Week-1){
            horLayoutWeek->addSpacing(45);
        }
    }

    verLayoutCalendar->addWidget(widgetWeek);

    widgetBody = new QWidget(this);
    widgetBody->setObjectName("widgetDay");

    verLayoutCalendar->addSpacing(2);

    gridLayoutBody = new QGridLayout(widgetBody);
    gridLayoutBody->setHorizontalSpacing(0);
    gridLayoutBody->setVerticalSpacing(0);
    gridLayoutBody->setContentsMargins(8, 0, 8, 0);

    for (int i = 0; i < 42; i++) {
        labelDay[i] = new DateLabel(widgetBody);
        connect(labelDay[i],SIGNAL(selectdate(QDate)),this,SLOT(PaintDate(QDate)));
        gridLayoutBody->addWidget(labelDay[i], i / 7, i % 7);
    }
    verLayoutCalendar->addWidget(widgetBody, 1);
    this->setLayout(verLayoutCalendar);

    selecteddate = QDate::currentDate();

    PaintToolDate(selecteddate);
}

void CalendarWidget::PaintToolDate(QDate date)
{
    selecteddate = date;
    int days = date.day();
    int weekday = date.dayOfWeek();
    int premonth_day;
    if (days < weekday) {
        premonth_day = weekday - days;
    } else {
         premonth_day = 7 - ((days-weekday) % 7);
    }

    if (premonth_day ==0 ){
        premonth_day = 7;
    }

    int from = 0-premonth_day-days+1;
    for (int i = 0 ,j = from; i < 42; i++,j++) {
        QDate d = date.addDays(j);
        if (d.month() != date.month()){
            labelDay[i]->setDate(d,date,true);
        } else {
            labelDay[i]->setDate(d,date,false);
        }
    }
}

void CalendarWidget::PaintDate(QDate date,bool refresh)
{
    if (refresh){
        emit settooldate(date);
        PaintToolDate(date);
    } else if (date.month() != selecteddate.month()){
        emit settooldate(date);
        PaintToolDate(date);
    }
}

void yearWidget::Init()
{

}
void yearWidget::inityear(int y)
{
    year = y;
    verLayoutCalendar = new QVBoxLayout();
    verLayoutCalendar->setSpacing(0);

    widgetBody = new QWidget(this);
    verLayoutCalendar->addWidget(widgetBody);

    gridLayoutBody = new QGridLayout(widgetBody);
    gridLayoutBody->setHorizontalSpacing(30);
    gridLayoutBody->setVerticalSpacing(0);

    int startyear = y;
    for (int i = 0; i < 16; i++) {

        labelyear[i] = new YearLabel(widgetBody);
        if (i > 9) {
            labelyear[i]->setyear(startyear+i,true);
        } else {
            labelyear[i]->setyear(startyear+i,false);
        }

        maxyear = startyear + i;
        connect(labelyear[i],SIGNAL(selectyear(int)),this,SLOT(getselectyear(int)));
        gridLayoutBody->addWidget(labelyear[i], i / 4, i % 4);
    }
    this->setLayout(verLayoutCalendar);

}

void yearWidget::paintEvent(QPaintEvent *event)
{

}

void monthWidget::Init()
{
    verLayoutCalendar = new QVBoxLayout();
    verLayoutCalendar->setSpacing(0);

    widgetBody = new QWidget(this);
    verLayoutCalendar->addWidget(widgetBody);

    gridLayoutBody = new QGridLayout(widgetBody);
    gridLayoutBody->setHorizontalSpacing(35);
    gridLayoutBody->setVerticalSpacing(0);
    gridLayoutBody->setContentsMargins(0, 0, 0, 0);
    QDate date = m_date.addMonths(1-m_date.month());
    int year = date.year();
    for (int i = 0; i < 16; i++) {
        labelDay[i] = new monthLabel(widgetBody);
        QDate tmp = date;
        tmp = tmp.addMonths(i);
        bool d;
        if (year == tmp.year()){
            d = false;
        } else {
            d = true;
        }
        labelDay[i]->setmonth(tmp,d);
        connect(labelDay[i],SIGNAL(selectmonth(QDate)),this,SLOT(getselectmonth(QDate)));
        gridLayoutBody->addWidget(labelDay[i], i / 4, i % 4);
    }
    this->setLayout(verLayoutCalendar);

}

void monthWidget::update(QDate d)
{
    m_date = d;
     QDate date = d.addMonths(1-m_date.month());
     int year = date.year();
    for (int i = 0; i < 16; i++) {
        QDate tmp = date;
        tmp = tmp.addMonths(i);
        bool d;
        if (year == tmp.year()){
            d = false;
        } else {
            d = true;
        }
        labelDay[i]->setmonth(tmp,d);
    }
    emit settooldate(m_date);


}


dateWidget::dateWidget(INdePanelPlugin * plugin,QWidget *parent)
    : QWidget(parent),
      mPlugin(plugin),
      mOpacity(-1)
{
    this->setWindowFlags(Qt::FramelessWindowHint |
                         Qt::Tool |
                         Qt::X11BypassWindowManagerHint);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);

    KWindowEffects::enableBlurBehind(this->winId(), true);
    setFixedSize(438,590);
    selectdirect = nullptr;

    Init();
}

bool dateWidget::event(QEvent *e)
{
    if(e->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            this->hide();
        }
    }
    return QWidget::event(e);
}
void dateWidget::clearlayout()
{
    QLayoutItem *child;
    while ((child = conlay->takeAt(1)) != 0)
    {
        if(child->widget())
        {
            child->widget()->setParent(NULL);
        }

        delete child;
    }
    if(conlay) {
        delete conlay;
    }
}

void dateWidget::Init()
{
    conlay = new QVBoxLayout();
    conlay->setSizeConstraint(QVBoxLayout::SetMinAndMaxSize);
    conlay->addSpacing(15);
    datetitle = new DateTitle(this);
    conlay->addWidget(datetitle);
    conlay->addSpacing(10);
    toolbar = new DateToolbar(m_date,false,this);
    conlay->addWidget(toolbar);
    conlay->addSpacing(20);
    clwidget = new CalendarWidget(this);
    conlay->addWidget(clwidget);
    this->setLayout(conlay);
    connect(toolbar,SIGNAL(setyear(int)),this,SLOT(setyear(int)));
    connect(toolbar,SIGNAL(setmonth(int)),this,SLOT(setmonth(int)));
    connect(toolbar,SIGNAL(ChangeDate(QDate)),clwidget,SLOT(PaintToolDate(QDate)));
    connect(toolbar,SIGNAL(Changetoday(QDate)),clwidget,SLOT(PaintToolDate(QDate)));
    connect(clwidget,SIGNAL(settooldate(QDate)),toolbar,SLOT(setdate(QDate)));
    connect(toolbar,SIGNAL(setCalendarDate(QDate)),this,SLOT(setCalendar(QDate)));
}


void dateWidget::setCalendar(QDate d)
{
    m_date = d;
    //datetitle = new DateTitle(this);
    toolbar = new DateToolbar(m_date,false,this);
    clwidget = new CalendarWidget(this);
    connect(toolbar,SIGNAL(setyear(int)),this,SLOT(setyear(int)));
    connect(toolbar,SIGNAL(setmonth(int)),this,SLOT(setmonth(int)));
    connect(toolbar,SIGNAL(ChangeDate(QDate)),clwidget,SLOT(PaintToolDate(QDate)));
    connect(toolbar,SIGNAL(Changetoday(QDate)),clwidget,SLOT(PaintToolDate(QDate)));
    connect(clwidget,SIGNAL(settooldate(QDate)),toolbar,SLOT(setdate(QDate)));
    connect(toolbar,SIGNAL(setCalendarDate(QDate)),this,SLOT(setCalendar(QDate)));
    clwidget->PaintDate(d,true);

    clearlayout();
    conlay = new QVBoxLayout();
    conlay->addSpacing(15);
    conlay->addWidget(datetitle);
    conlay->addSpacing(10);
    conlay->addWidget(toolbar);
    conlay->addSpacing(20);
    conlay->addWidget(clwidget);
    setLayout(conlay);
}

void dateWidget::realign()
{
    //检测配置文件中的透明度变化
    int opacity = mPlugin->panel()->opacity();
    QColor bColor = mPlugin->panel()->backgroundColor();

    if(mOpacity == opacity && mBackgroundColor == bColor )
        return;

    mOpacity = opacity;
    mBackgroundColor = bColor;
    QString color = QString("%1, %2, %3, %4")
        .arg(mBackgroundColor.red())
        .arg(mBackgroundColor.green())
        .arg(mBackgroundColor.blue())
        .arg(mOpacity / 100.0);
    this->setStyleSheet(QString("dateWidget {background-color: rgba(" + color + "); }"));
}

void dateWidget::setyear(int year)
{

    //datetitle = new DateTitle(this);
    toolbar = new DateToolbar(m_date,true,this);
    toolbar->Inityear();
    //toolbar->setselectyear();
    ywidget = new yearWidget(this);
    ywidget->inityear(year);
    connect(toolbar,SIGNAL(setmonth(int)),this,SLOT(setmonth(int)));
    connect(ywidget,SIGNAL(setselectyear(int)),toolbar,SLOT(getyear(int)));
    connect(ywidget,SIGNAL(settoolyear(int)),toolbar,SLOT(showyear(int)));
    connect(toolbar,SIGNAL(setCalendarDate(QDate)),this,SLOT(setCalendar(QDate)));
    connect(toolbar,SIGNAL(Changetoday(QDate)),this,SLOT(setCalendar(QDate)));
    connect(toolbar,SIGNAL(ChangeDate(QDate)),ywidget,SLOT(ChangeNewDate(QDate)));

    clearlayout();
    conlay = new QVBoxLayout();
    conlay->addSpacing(15);
    conlay->addWidget(datetitle);
    conlay->addSpacing(10);
    conlay->addWidget(toolbar);
    conlay->addSpacing(20);
    conlay->addWidget(ywidget);
    setLayout(conlay);
}

void dateWidget::setmonth(int year)
{
    //datetitle = new DateTitle(this);
    toolbar = new DateToolbar(m_date,true,this);
    toolbar->Initmonth();
    //toolbar->setselectmonth();
    mwidget = new monthWidget(m_date,this);
    connect(toolbar,SIGNAL(setyear(int)),this,SLOT(setyear(int)));
    connect(mwidget,SIGNAL(settoolmonth(QDate)),toolbar,SLOT(getmonth(QDate)));
    connect(mwidget,SIGNAL(settooldate(QDate)),toolbar,SLOT(showmonth(QDate)));
    connect(toolbar,SIGNAL(setCalendarDate(QDate)),this,SLOT(setCalendar(QDate)));
    connect(toolbar,SIGNAL(Changetoday(QDate)),this,SLOT(setCalendar(QDate)));
    connect(toolbar,SIGNAL(ChangeDate(QDate)),mwidget,SLOT(ChangeNewDate(QDate)));

    clearlayout();
    conlay = new QVBoxLayout();
    conlay->addSpacing(15);
    conlay->addWidget(datetitle);
    conlay->addSpacing(10);
    conlay->addWidget(toolbar);
    conlay->addSpacing(20);
    conlay->addWidget(mwidget);
    setLayout(conlay);
}


void dateWidget::paintEvent(QPaintEvent *event)
{
    QRect rect  = this->rect().adjusted(0,0,-1,-1);
    QBitmap bmp(rect.size());
    bmp.fill();
    QPainter painter(&bmp);
    painter.setBrush(Qt::black);
    painter.setRenderHints(QPainter::HighQualityAntialiasing);
    painter.drawRoundedRect(bmp.rect(),10,10);
    setMask(bmp);
}


dateWidget::~dateWidget()
{

}
