#ifndef CALENDARWIDGET_H
#define CALENDARWIDGET_H
#include "../panel/indepanelplugin.h"
#include <QWidget>
#include <QVBoxLayout>
#include <QLabel>
#include <QDate>
#include <QHash>
#include <QDebug>
#include <QEvent>
#include <Nde/RotatedWidget>
#include <QApplication>
#include <Nde/Settings>
#include <QPushButton>
#include <QWheelEvent>
typedef enum {
    Sunday,
    Monday,
    Tuesday,
    Wednesday,
    Thursday,
    Friday,
    Saturday,
    Week
} E_WEEK;

static const QString s_strWeek[] = {
    QObject::tr("一"), QObject::tr("二"),QObject::tr("三"),
    QObject::tr("四"), QObject::tr("五"),QObject::tr("六"),QObject::tr("日"),
};

static const QString s_strWeekl[] = {
    QObject::tr("周一"), QObject::tr("周二"),QObject::tr("周三"),
    QObject::tr("周四"), QObject::tr("周五"),QObject::tr("周六"),QObject::tr("周日"),
};



class Date : public QObject
{
    Q_OBJECT
public:
    explicit Date(QObject *parent = 0);
public:
    static QString getLunarDate(int year, int month, int day);
};

class DateConfig: public QObject
{
    Q_OBJECT
public:
    static DateConfig& instance();
    int getDate(QDate date);
private:
    DateConfig() {
        Init();
    }
    void Init();
private:
    QHash<QString,int> worklist;
};

class YearLabel : public QWidget
{
    Q_OBJECT
public:
    YearLabel(QWidget*parent = nullptr) : QWidget(parent){
        setFixedSize(60,60);
        lab_year = new QLabel;
        lab_year->setFixedWidth(60);
        lab_year->setStyleSheet("font-size:16px;color:rgb(255,255,255)");
    }
    void setyear(int y,bool d) {
        year = y;
        dark = d;
        Refresh();
    }

signals:
    void selectyear(int);
protected:

    virtual void mousePressEvent(QMouseEvent *event){
        emit selectyear(year);
    }
    virtual void paintEvent(QPaintEvent *event);
    virtual void enterEvent(QEvent *event){
        moveon = true;
        repaint();
    }
    virtual void leaveEvent(QEvent *event){
        moveon = false;
        repaint();
    }
private:

    void Refresh() {
        if(verLayoutdate){
            delete verLayoutdate;
        }
        if (dark) {
            lab_year->setStyleSheet("dateWidget QLabel{font-size:16px;color:rgba(255,255,255,100)}");
        } else {
            lab_year->setStyleSheet("dateWidget QLabel{font-size:16px;color:rgb(255,255,255)}");
        }
        lab_year->setText(QString::number(year) + "年");
        verLayoutdate = new QVBoxLayout(this);
        verLayoutdate->setAlignment(Qt::AlignCenter);
        verLayoutdate->addWidget(lab_year);
        repaint();
    }

private:
    QVBoxLayout *verLayoutdate {nullptr};
    QLabel * lab_year;
    int year {2020};
    bool dark;
    bool moveon {false};
};


class monthLabel : public QWidget
{
    Q_OBJECT
public:
    monthLabel(QWidget*parent = nullptr) : QWidget(parent){

        setFixedSize(60,60);
        lab_month = new QLabel;
        lab_month->setStyleSheet("font-size:16px;color:rgb(255,255,255)");
    }
    void setmonth(QDate y,bool d = false) {
        dark = d;
        month = y;
        Refresh();
    }
signals:
    void selectmonth(QDate);
protected:
    virtual void mousePressEvent(QMouseEvent *event){
        emit selectmonth(month);
    }
    virtual void paintEvent(QPaintEvent *event);
    virtual void enterEvent(QEvent *event){
        moveon = true;
        repaint();
    }
    virtual void leaveEvent(QEvent *event){
        moveon = false;
        repaint();
    }
private:

    void Refresh() {
        if(verLayoutdate){
            delete verLayoutdate;
        }

        lab_month->setText(QString::number(month.month()) + "月");
        if (dark) {
            lab_month->setStyleSheet("dateWidget QLabel{font-size:16px;color:rgba(255,255,255,100)}");
        } else {
            lab_month->setStyleSheet("dateWidget QLabel{font-size:16px;color:rgb(255,255,255)}");
        }
        verLayoutdate = new QVBoxLayout(this);
        verLayoutdate->setAlignment(Qt::AlignCenter);
        verLayoutdate->addWidget(lab_month);
        repaint();
    }
private:
    QVBoxLayout *verLayoutdate {nullptr};
    QLabel * lab_month;
    QDate month {QDate::currentDate()};
    bool moveon {false};
    bool dark {false};
};

class DateLabel : public QWidget
{
    Q_OBJECT
public:
    DateLabel(QWidget * parent = nullptr);
    void setDate(QDate date,QDate curdate,bool dark);
    void setSelect(bool b){
        bselected = b;
        repaint();
    }
signals:
    void selectdate(QDate);

protected:
    virtual void paintEvent(QPaintEvent *event);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void enterEvent(QEvent *event){
        moveon = true;
        repaint();
    }
    virtual void leaveEvent(QEvent *event){
        moveon = false;
        repaint();
    }
private:
    QDate m_date;
    QLabel * new_date;
    QLabel * old_date;
    QVBoxLayout *verLayoutdate {nullptr};
    bool bselected {false};
    bool bcurday {false};
    bool moveon {false};
    bool bwork {true};
    bool bholiday{false};
    bool dark {false};

};


class YMLabel : public QPushButton
{
    Q_OBJECT
public:
    YMLabel(QWidget * parent = nullptr) : QPushButton(parent){
        setStyleSheet("dateWidget QPushButton{background: transparent;border:none;font-size:18px;color:#ffffff}"
                      "dateWidget QPushButton:hover{background: transparent;font-size:18px;color:#2c9dfd}"
                      "dateWidget QPushButton:pressed{background: transparent;font-size:18px;color:#0082f0}");
    }
    void setSelected(bool selected = true);
signals:
    void clicked();
protected:
    void mousePressEvent(QMouseEvent * event);

};

class todyLabel : public QPushButton
{
    Q_OBJECT
public:
    todyLabel(QWidget * parent = nullptr) : QPushButton(parent){
        setFixedWidth(60);
        setFlat(true);
        setStyleSheet("dateWidget QPushButton{background: transparent;border:none;font-size:14px;color:#2c9dfd}"
                      "dateWidget QPushButton:hover{background: transparent;font-size:14px;color:rgba(255,255,255,100)}"
                      "dateWidget QPushButton:pressed{background: transparent;font-size:14px;color:rgba(255,255,255,50)}");

    }
private slots:

signals:
    void clicked();

protected:
    void mousePressEvent(QMouseEvent * event){
        setFlat(true);
        setStyleSheet("border:none;font-size:14px;color:rgba(255,255,255,50)");
        emit clicked();
    }
    void mouseReleaseEvent(QMouseEvent * event){
        setFlat(true);
        setStyleSheet("background: transparent;border:none;font-size:14px;color:#2c9dfd");
    }
    virtual void enterEvent(QEvent *event){
        setFlat(true);
        setStyleSheet("background: transparent;border:none;font-size:14px;color:rgba(255,255,255,100)");
    }
    virtual void leaveEvent(QEvent *event){
        setFlat(true);
        setStyleSheet("background: transparent;border:none;font-size:14px;color:#2c9dfd");
    }

};

class DirectWidget : public QWidget
{
    Q_OBJECT
public:
    DirectWidget(int Direct,QWidget* parent = nullptr) :
        QWidget(parent)
    {
        setFixedSize(10,18);
        direction = Direct;
        selected = false;
    }

    void setSelected(bool f);

signals:
    void clicked();

protected:
    void mousePressEvent(QMouseEvent * event);
    void mouseReleaseEvent(QMouseEvent * event){
        selected = false;
        repaint();
    }
    virtual void paintEvent(QPaintEvent *event);
    virtual void enterEvent(QEvent *event){
        hover = true;
        repaint();
    }
    virtual void leaveEvent(QEvent *event){
        hover = false;

        repaint();

    }
private:
    int direction;
    bool selected;
    bool hover {false};
};


class DateTitle :public QWidget
{
    Q_OBJECT
public:
    DateTitle(QWidget * parent = nullptr);
protected:
    virtual void paintEvent(QPaintEvent *event);
private slots:
    void DateTimer();
private:
    void Init();
    void InitTimer();
    void SetDateTime();
private:
    QVBoxLayout *verLayouttime;
    QLabel * lab_time;
    QLabel * lab_date;
    QTimer * timer {nullptr};
};

class DateToolbar : public QWidget
{
    Q_OBJECT
public:
    DateToolbar(QDate d,bool b_showym = false,QWidget* parent = nullptr);
    void setselectyear(){
        cur_year->setSelected();
        //repaint();
    }
    void setselectmonth(){
        cur_month->setSelected();
        //repaint();
    }
    void Inityear(){
        showyear(m_date.year());
    }

    void Initmonth(){
        showmonth(m_date);
    }
signals:
    void ChangeDate(QDate);
    void Changetoday(QDate);
    void setCalendarDate(QDate);
    void setyear(int);
    void setmonth(int);

protected:
    virtual void paintEvent(QPaintEvent *event);


private slots:
    void ontoday() {
        m_date = QDate::currentDate();
        emit Changetoday(m_date);
        repaint();
    }
    void preyear() {
        m_date = m_date.addYears(-1);
        emit ChangeDate(m_date);
        cur_month->setSelected(false);
        repaint();
    }
    void curyear() {
        emit setyear(m_date.year());

    }
    void nextyear() {
        m_date = m_date.addYears(1);
        emit ChangeDate(m_date);
        cur_month->setSelected(false);
        repaint();
    }
    void premonth() {
        m_date = m_date.addMonths(-1);
        emit ChangeDate(m_date);
        cur_year->setSelected(false);
        repaint();
    }
    void curmonth() {
        emit setmonth(m_date.month());

    }
    void nextmonth() {
        m_date = m_date.addMonths(1);
        emit ChangeDate(m_date);
        cur_year->setSelected(false);
        repaint();
    }

    void setdate(QDate d){
        m_date = d;
        repaint();
    }

    void getyear(int y){
        m_date.setDate(y,m_date.month(),m_date.day());
        emit setCalendarDate(m_date);
        repaint();
    }

    void showyear(int y){
        QString str = QString::number(y) + "-" + QString::number(y+9) + "年";
        showym->setText(str);
        //repaint();
    }

    void getmonth(QDate m){
        m_date.setDate(m.year(),m.month(),m_date.day());
        emit setCalendarDate(m_date);
        repaint();
    }
    void showmonth(QDate d){
        QString str = QString::number(d.year()) + "年" + QString::number(d.month()) + "月";
        showym->setText(str);
        //repaint();
    }

private:
    void Init();
    void paintitem();
private:

    QDate m_date;

    QHBoxLayout * toolbarlay;

    DirectWidget * pre_year;
    YMLabel * cur_year;
    DirectWidget * next_year;

    DirectWidget * pre_month;
    YMLabel * cur_month;
    DirectWidget * next_month;

    todyLabel * today;

    QLabel * showym;
    bool b_showym;
};

class CalendarWidget : public QWidget
{
    Q_OBJECT
public:
    explicit CalendarWidget(QWidget *parent = nullptr);

protected:
    virtual void wheelEvent(QWheelEvent *event){
        if(event->delta() > 0){
            selecteddate = selecteddate.addMonths(-1);
            PaintDate(selecteddate,true);
        } else {
            selecteddate = selecteddate.addMonths(1);
            PaintDate(selecteddate,true);
        }
    }

signals:
    void settooldate(QDate);
public slots:
    void PaintDate(QDate date,bool refresh = false);
    void PaintToolDate(QDate date);
private:
    void Init();
private:
    QVBoxLayout *verLayoutCalendar;

    QWidget *widgetWeek;
    QLabel * labweek[Week];
    QHBoxLayout *horLayoutWeek;


    QWidget *widgetBody;
    DateLabel *labelDay[42];
    QGridLayout *gridLayoutBody;

    QDate selecteddate;
};

class yearWidget : public QWidget
{
    Q_OBJECT
public:
    yearWidget(QWidget * parent = nullptr): QWidget(parent){
        setFixedSize(438,440);
        //Init();
    }
    ~yearWidget(){}
    void inityear(int);
signals:
    void settoolyear(int);
    void setselectyear(int);

protected:
    virtual void wheelEvent(QWheelEvent *event){
        if(event->delta() > 0){
            year = year - 10;
        } else {
            year = year + 10;
        }
        update(year);
    }

    virtual void paintEvent(QPaintEvent *event);

public slots:
    void getselectyear(int y) {
        emit setselectyear(y);
    }
    void ChangeNewDate(QDate date){
        if (date.year()<year){
            update(date.year() - 15);
        } else if (date.year() > maxyear) {
            update(date.year());
        }
    }
private:
    void Init();
    void update(int y){
        year = y;
        int startyear = year;
        for (int i = 0; i < 16; i++) {
            if (i > 9) {
                labelyear[i]->setyear(startyear+i,true);
            } else {
                labelyear[i]->setyear(startyear+i,false);
            }

            maxyear = startyear + i ;
        }
        emit settoolyear(year);
    }
private:
    YearLabel *labelyear[16];
    QVBoxLayout *verLayoutCalendar;
    QWidget *widgetBody;
    QGridLayout *gridLayoutBody;
    int year;
    int maxyear;
};

class monthWidget : public QWidget
{
    Q_OBJECT
public:
    monthWidget(QDate d,QWidget * parent = nullptr): QWidget(parent){
        m_date = d;
        setFixedSize(438,440);
        Init();
    }
    ~monthWidget(){}

signals:
    void settoolmonth(QDate);
    void settooldate(QDate);
protected:
    virtual void wheelEvent(QWheelEvent *event){
        if(event->delta() > 0){
            m_date = m_date.addMonths(-12);
        } else {
            m_date = m_date.addMonths(12);
        }
        update(m_date);

    }
public slots:
    void getselectmonth(QDate y) {
        emit settoolmonth(y);
    }
    void ChangeNewDate(QDate d) {
        QDate maxdate = m_date.addMonths(15);
        if ((d.year() < m_date.year()) || ((d.year() == m_date.year()) && (d.month() < m_date.month()))){
            update(d.addMonths(-15));
        } else if ((d.year() == maxdate.year())  && (d.month() > maxdate.month())) {
            update(d);
        }
    }
private:
    void Init();
    void update(QDate);
private:
    monthLabel *labelDay[16];
    QVBoxLayout *verLayoutCalendar;
    QWidget *widgetBody;
    QGridLayout *gridLayoutBody;
    QDate m_date;
};


class dateWidget : public QWidget
{
    Q_OBJECT

public:
    dateWidget(INdePanelPlugin * plugin,QWidget *parent = nullptr);
    ~dateWidget();

    void setcurdate(){
        setCalendar(QDate::currentDate());
    }

    void realign();

public slots:
    void setyear(int);
    void setmonth(int);
    void setCalendar(QDate);

protected:
    bool event(QEvent *e);
    void paintEvent(QPaintEvent *event);
private:
    void Init();
    void clearlayout();
private:
    INdePanelPlugin * mPlugin;
    int mOpacity; //背景透明度
    QColor mBackgroundColor; //背景色

    QVBoxLayout * conlay {nullptr};
    DateTitle * datetitle;
    DateToolbar * toolbar;
    CalendarWidget * clwidget;
    yearWidget * ywidget;
    monthWidget * mwidget;

    QDate m_date {QDate::currentDate()};
};


#endif // CALENDARWIDGET_H
